package com.tzld.piaoquan.longarticle.config;

import com.alibaba.druid.pool.DruidDataSource;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.transaction.PlatformTransactionManager;

import javax.sql.DataSource;

@MapperScan(basePackages = CrawlerDataSourceConfig.CRAWLER_PACKAGE_MASTER, sqlSessionFactoryRef = "crawlerSqlSessionFactory")
@Configuration
public class CrawlerDataSourceConfig {



    // 数据源
    static final String CRAWLER_PACKAGE_MASTER = "com.tzld.piaoquan.longarticle.dao.mapper.crawler";
    static final String CRAWLER_MAPPER_LOCATION_MASTER = "classpath:mapper/crawler/*.xml";

    @Bean(name = "crawlerDataSource")
    @ConfigurationProperties("crawler.datasource")
    public DataSource getCrawlerDataSource(){
        return new DruidDataSource();
    }

    @Bean(name = "crawlerTransactionManager")
    public PlatformTransactionManager crawlerTransactionManager(@Qualifier("crawlerDataSource") DataSource dataSource) {
        return new DataSourceTransactionManager(dataSource);
    }

    @Bean(name = "crawlerSqlSessionFactory")
    public SqlSessionFactory crawlerSqlSessionFactory(@Qualifier("crawlerDataSource") DataSource dataSource) throws Exception {
        final SqlSessionFactoryBean sessionFactoryBean = new SqlSessionFactoryBean();
        sessionFactoryBean.setDataSource(dataSource);
        sessionFactoryBean.setMapperLocations(new PathMatchingResourcePatternResolver().getResources(CrawlerDataSourceConfig.CRAWLER_MAPPER_LOCATION_MASTER));
        return sessionFactoryBean.getObject();
    }


}
