package com.tzld.piaoquan.longarticle.service.local.impl;

import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.AccountSpecialSettingsMapper;
import com.tzld.piaoquan.longarticle.model.po.longarticle.AccountSpecialSettings;
import com.tzld.piaoquan.longarticle.model.po.longarticle.AccountSpecialSettingsExample;
import com.tzld.piaoquan.longarticle.service.local.AccountService;
import com.tzld.piaoquan.longarticle.utils.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class AccountServiceImpl implements AccountService {

    @Autowired
    private AccountSpecialSettingsMapper accountSpecialSettingsMapper;

    @ApolloJsonValue("${specialBeforeDays:{}}")
    private Map<String, Integer> specialBeforeDaysMap;

    @Override
    public List<AccountSpecialSettings> getAccountSpecialSettings() {
        String dateString = DateUtil.getThatDayDateString();
        AccountSpecialSettingsExample example = new AccountSpecialSettingsExample();
        example.createCriteria().andEffectiveDateEqualTo(dateString).andIsDeleteEqualTo(0)
                .andOpenStatusEqualTo(1);
        return accountSpecialSettingsMapper.selectByExample(example);
    }

    public List<AccountSpecialSettings> getSendAccountSpecialSettings() {
        String dateString = DateUtil.getTomorrowDateString();
        AccountSpecialSettingsExample example = new AccountSpecialSettingsExample();
        example.createCriteria().andEffectiveDateEqualTo(dateString).andIsDeleteEqualTo(0)
                .andOpenStatusEqualTo(1);
        List<AccountSpecialSettings> accountSpecialSettings = accountSpecialSettingsMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(accountSpecialSettings)) {
            accountSpecialSettings = new ArrayList<>();
        }
        accountSpecialSettings = accountSpecialSettings.stream().filter(f -> !specialBeforeDaysMap.containsKey(f.getSpecialSettingId())).collect(Collectors.toList());
        for (Map.Entry<String, Integer> entry : specialBeforeDaysMap.entrySet()) {
            dateString = DateUtil.getAfterDaysDateString(entry.getValue());
            AccountSpecialSettingsExample accountSpecialSettingsExample = new AccountSpecialSettingsExample();
            accountSpecialSettingsExample.createCriteria().andEffectiveDateEqualTo(dateString).andIsDeleteEqualTo(0)
                    .andOpenStatusEqualTo(1).andSpecialSettingIdEqualTo(entry.getKey());
            List<AccountSpecialSettings> accountSpecialSettingsList = accountSpecialSettingsMapper.selectByExample(accountSpecialSettingsExample);
            if (!CollectionUtils.isEmpty(accountSpecialSettingsList)) {
                accountSpecialSettings.addAll(accountSpecialSettingsList);
            }

        }
        return accountSpecialSettings;
    }

    public void saveOrUpdateAccountSpecialSettings(AccountSpecialSettings accountSpecialSettings) {
        AccountSpecialSettingsExample example = new AccountSpecialSettingsExample();
        example.createCriteria().andAccountIdEqualTo(accountSpecialSettings.getAccountId())
                .andSpecialSettingIdEqualTo(accountSpecialSettings.getSpecialSettingId());
        List<AccountSpecialSettings> accountSpecialSettingsList = accountSpecialSettingsMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(accountSpecialSettingsList)) {
            accountSpecialSettingsMapper.insertSelective(accountSpecialSettings);
        } else {
            accountSpecialSettings.setId(accountSpecialSettingsList.get(0).getId());
            accountSpecialSettings.setIsDelete(0);
            accountSpecialSettingsMapper.updateByPrimaryKeySelective(accountSpecialSettings);
        }
    }

    @Override
    public boolean isSendSpecialSettings(String accountId) {
        List<AccountSpecialSettings> sendAccountSpecialSettings = getSendAccountSpecialSettings();
        if (CollectionUtils.isEmpty(sendAccountSpecialSettings)) {
            return false;
        }
        return sendAccountSpecialSettings.stream().anyMatch(settings -> settings.getAccountId().equals(accountId));
    }

    @Override
    public List<AccountSpecialSettings> getAccountSpecialSettingsById(String specialSettingsId) {
        AccountSpecialSettingsExample example = new AccountSpecialSettingsExample();
        example.createCriteria().andSpecialSettingIdEqualTo(specialSettingsId).andIsDeleteEqualTo(0);
        return accountSpecialSettingsMapper.selectByExample(example);
    }

    @Override
    public void delAccountSpecialSettings(Long id) {
        AccountSpecialSettings updateAccountSpecialSettings = new AccountSpecialSettings();
        updateAccountSpecialSettings.setId(id);
        updateAccountSpecialSettings.setIsDelete(1);
        accountSpecialSettingsMapper.updateByPrimaryKeySelective(updateAccountSpecialSettings);
    }


}
