package com.tzld.piaoquan.longarticle.service.local.impl;

import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.longarticle.common.enums.ContentStatusEnum;
import com.tzld.piaoquan.longarticle.common.enums.MatchResultStatusEnum;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.NewVideoCoverMapper;
import com.tzld.piaoquan.longarticle.model.bo.VideoDetail;
import com.tzld.piaoquan.longarticle.model.po.longarticle.*;
import com.tzld.piaoquan.longarticle.model.vo.MatchMiniprogramStatusParam;
import com.tzld.piaoquan.longarticle.service.local.CardService;
import com.tzld.piaoquan.longarticle.service.remote.AigcService;
import com.tzld.piaoquan.longarticle.utils.ImageUrlValidator;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

@Slf4j
@Service
public class CardServiceImpl implements CardService {

    @ApolloJsonValue("${miniMap:{}}")
    private JSONObject miniProgramMap;

    @Autowired
    AigcService aigcService;

    @Autowired
    private NewVideoCoverMapper newVideoCoverMapper;

    private static final List<String> crawlerImgPathList = new ArrayList<String>() {{
        add("20250208065525959973866");
        add("20250208111811026884434");
        add("20250212125646224172850");
        add("20250213023808593788754");
        add("20250213091337867990118");
        add("20250214091843879629665");
        add("20250214122110960857913");
        add("20250217080347557694800");
        add("20250217120153840400833");
        add("20250304073234245880032");
    }};

    private static final List<String> useShareImgPathList = new ArrayList<String>(){{
        add("妙用俏生活");
        add("票圈精彩");
    }};

    private static final List<String> useSmallPlayImgPathList = new ArrayList<String>();


    public JSONObject createGzhPath(String videoId, String sharedUid, String ghId, Integer miniProgramType) throws UnsupportedEncodingException {
        String rootShareId = UUID.randomUUID().toString();
        String rootSourceId;
        switch (miniProgramType) {
            case 1:
                rootSourceId = "longArticles_" + generateSourceId();
                break;
            case 2:
                rootSourceId = "touliu_tencentGzhArticle_" + ghId + "_" + generateSourceId();
                break;
            case 3:
                rootSourceId = "WeCom_" + generateSourceId();
                break;
            case 4:
                rootSourceId = "DaiTou_" + ghId + "_" + generateSourceId();
                break;
            case 5:
                rootSourceId = "gzhhzdx_" + generateSourceId();
                break;
            case 6:
                rootSourceId = "fwhhzdyy_" + generateSourceId();
                break;
            case 7:
                rootSourceId = "fwhtouliu_" + generateSourceId();
                break;
            case 8:
                rootSourceId = "fwhmh_" + generateSourceId();
                break;
            default:
                rootSourceId = "Error mini_program_type " + miniProgramType;
        }

        String url = String.format("pages/user-videos?id=%s&su=%s&fromGzh=1&rootShareId=%s&shareId=%s&rootSourceId=%s",
                videoId, sharedUid, rootShareId, rootShareId, rootSourceId);
        String productionPath = String.format("pages/category?jumpPage=%s", URLEncoder.encode(url, StandardCharsets.UTF_8.toString()));
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("rootShareId", rootShareId);
        jsonObject.put("rootSourceId", rootSourceId);
        jsonObject.put("productionPath", productionPath);
        return jsonObject;
    }

    private String generateSourceId() {
        long timestamp = System.currentTimeMillis();
        int randomNum = new Random().nextInt(9000) + 1000;
        String hashInput = timestamp + "-" + randomNum;
        return md5(hashInput);
    }

    private String md5(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] messageDigest = md.digest(input.getBytes(StandardCharsets.UTF_8));
            StringBuilder sb = new StringBuilder();
            for (byte b : messageDigest) {
                sb.append(String.format("%02x", b));
            }
            return sb.toString();
        } catch (NoSuchAlgorithmException e) {
            log.error("CardServiceImpl md5 error", e);
        }
        return null;
    }

    public PublishMiniprogram generateSingleCard(Integer index, Integer miniId, VideoDetail videoDetail,
                                                 PlanAccount planAccount, PublishContent publishContent) throws UnsupportedEncodingException {
        String strMiniId = String.valueOf(miniId);
        JSONObject miniInfo = miniProgramMap.getJSONObject(strMiniId);
        String avatar = miniInfo.getString("avatar");
        String appId = miniInfo.getString("id");
        String appName = miniInfo.getString("name");

        String videoId = videoDetail.getVideoId();
        String uid = videoDetail.getUid();
        JSONObject jsonObject = createGzhPath(videoId, uid, planAccount.getGhId(), planAccount.getMiniprogramUseType());

        PublishMiniprogram publishMiniprogram = new PublishMiniprogram();
        publishMiniprogram.setContentId(publishContent.getId());
        publishMiniprogram.setPlanAccountId(planAccount.getId());
        publishMiniprogram.setAppId(appId);
        publishMiniprogram.setPublishContentId(publishContent.getPublishContentId());
        publishMiniprogram.setAppName(appName);
        publishMiniprogram.setAvatar(avatar);
        publishMiniprogram.setMiniId(miniId);
        publishMiniprogram.setMiniProgramType(planAccount.getMiniprogramUseType());
        publishMiniprogram.setVideoId(Long.parseLong(videoId));
        publishMiniprogram.setSource(videoDetail.getSource());
        publishMiniprogram.setVideoTitle(videoDetail.getKimiTitle());
        if (crawlerImgPathList.contains(planAccount.getPlanId())
                || useShareImgPathList.contains(planAccount.getAccountName())
                || useSmallPlayImgPathList.contains(planAccount.getAccountName())) {
            publishMiniprogram.setVideoCover(videoDetail.getShareImgPath());
            if (useSmallPlayImgPathList.contains(planAccount.getAccountName())) {
                publishMiniprogram.setVideoCover(videoDetail.getSmallPlayImgPath());
            }
            if (crawlerImgPathList.contains(planAccount.getPlanId())) {
                publishMiniprogram.setVideoCover(videoDetail.getRawImgPath());
            }
        } else {
            publishMiniprogram.setVideoCover(videoDetail.getCoverImgPath());
        }
        publishMiniprogram.setVideoPath(videoDetail.getVideoPath());
        publishMiniprogram.setProductionPath(jsonObject.getString("productionPath"));
        publishMiniprogram.setRootSourceId(jsonObject.getString("rootSourceId"));
        publishMiniprogram.setRootShareId(jsonObject.getString("rootShareId"));
        publishMiniprogram.setIndex(index);
        publishMiniprogram.setTraceId(videoDetail.getTraceId());
        publishMiniprogram.setCrawlerVideoId(videoDetail.getCrawlerVideoId());
        publishMiniprogram.setVideoOssPath(videoDetail.getVideoOss());
        publishMiniprogram.setUserId(videoDetail.getUid());
        return publishMiniprogram;
    }

    public List<PublishMiniprogram> generateCards(List<VideoDetail> videoDetails, PlanAccount planAccount, PublishContent publishContent) {
        try {
            int longArticlesMiniProgramId = 25;
            int touliuMiniProgramId = 33;
            int weComMiniProgramId = 27;
            int daiTouMiniProgramId = 25;
            int gzhhzdxMiniProgramId = 25;
            int fwhhzdyyMiniProgramId = 25;
            int fwhtouMiniProgramId = 25;
            int fwhmhMiniProgramId = 25;
            int miniId = 0;
            List<PublishMiniprogram> cardList = new ArrayList<>();
            switch (planAccount.getMiniprogramUseType()) {
                case 1:
                    miniId = longArticlesMiniProgramId;
                    break;
                case 2:
                    miniId = touliuMiniProgramId;
                    break;
                case 3:
                    miniId = weComMiniProgramId;
                    break;
                case 4:
                    miniId = daiTouMiniProgramId;
                    break;
                case 5:
                    miniId = gzhhzdxMiniProgramId;
                    break;
                case 6:
                    miniId = fwhhzdyyMiniProgramId;
                    break;
                case 7:
                    miniId = fwhtouMiniProgramId;
                    break;
                case 8:
                    miniId = fwhmhMiniProgramId;
                    break;
                default:
                    break;
            }
            if (miniId == 0) {
                return null;
            }
            for (int index = 0; index < videoDetails.size(); index++) {
                VideoDetail videoDetail = videoDetails.get(index);
                PublishMiniprogram card = generateSingleCard(index + 1, miniId, videoDetail, planAccount, publishContent);
//                //1号位置和2号位置视频  查询2号视频是否有新封面
//                if (index == 1 && Objects.equals(videoDetail.getCrawlerVideoId(), videoDetails.get(0).getCrawlerVideoId())) {
//                    NewVideoCoverExample example = new NewVideoCoverExample();
//                    example.createCriteria().andCrawlerVideoIdEqualTo(videoDetail.getCrawlerVideoId()).andGetCoverStatusEqualTo(2);
//                    List<NewVideoCover> newVideoCovers = newVideoCoverMapper.selectByExample(example);
//                    // 查询是否存在新封面
//                    if (!CollectionUtils.isEmpty(newVideoCovers)) {
//                        NewVideoCover newVideoCover = newVideoCovers.get(0);
//                        String cover = " https://rescdn.yishihui.com/" + newVideoCover.getCoverOssPath();
//                        card.setVideoCover(cover);
//                        card.setNewVideoCoverId(newVideoCover.getId());
//                    }
//                }
                if (card != null) {
                    String videoCover = card.getVideoCover();
                    String wxUrl = aigcService.pushCover(videoCover, card.getPublishContentId());
                    if (StringUtils.isEmpty(wxUrl)) {
                        boolean validImageUrl = ImageUrlValidator.isValidImageUrl(videoCover);
                        if (!validImageUrl) {
                            MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                            statusParam.setStatus(MatchResultStatusEnum.FAIL.getStatusCode());
                            statusParam.setPublishContentId(card.getPublishContentId());
                            statusParam.setErrorMsg("封面不可用");
                            aigcService.updateMatchMiniprogramStatus(statusParam);
                        }
                        continue;
                    }
                    card.setWxUrl(wxUrl);
                    cardList.add(card);
                }
            }
            return cardList;
        } catch (UnsupportedEncodingException e) {
            log.error("generateCards error {}", e.getMessage());
        }
        return null;
    }
}
