package com.tzld.piaoquan.longarticle.service.local.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.huaban.analysis.jieba.JiebaSegmenter;
import com.tzld.piaoquan.longarticle.common.enums.*;
import com.tzld.piaoquan.longarticle.dao.mapper.growth.ContentPlatformVideoMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.*;
import com.tzld.piaoquan.longarticle.model.bo.VideoDetail;
import com.tzld.piaoquan.longarticle.model.dto.ArticleSortRequest;
import com.tzld.piaoquan.longarticle.model.dto.ArticleSortResponse;
import com.tzld.piaoquan.longarticle.model.dto.ArticleSortResponseData;
import com.tzld.piaoquan.longarticle.model.dto.PublishArticleData;
import com.tzld.piaoquan.longarticle.model.po.growth.ContentPlatformVideo;
import com.tzld.piaoquan.longarticle.model.po.growth.ContentPlatformVideoExample;
import com.tzld.piaoquan.longarticle.model.po.longarticle.*;
import com.tzld.piaoquan.longarticle.model.vo.*;
import com.tzld.piaoquan.longarticle.service.local.ContentService;
import com.tzld.piaoquan.longarticle.service.local.KimiService;
import com.tzld.piaoquan.longarticle.service.remote.AigcService;
import com.tzld.piaoquan.longarticle.service.remote.MatchService;
import com.tzld.piaoquan.longarticle.service.remote.SortService;
import com.tzld.piaoquan.longarticle.service.remote.VideoService;
import com.tzld.piaoquan.longarticle.utils.DateUtil;
import com.tzld.piaoquan.longarticle.utils.LarkRobotUtil;
import com.tzld.piaoquan.longarticle.utils.ToolUtil;
import com.tzld.piaoquan.longarticle.utils.VideoUtils;
import com.tzld.piaoquan.longarticle.utils.other.NlpUtils;
import com.tzld.piaoquan.longarticle.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ContentServiceImpl implements ContentService {
    @ApolloJsonValue("${testAccountLevel2:{}}")
    private Map<String, Integer> ghIdMap;

    private Set<String> ghIdSet = new HashSet<String>();

    private Set<String> accountNameSet = new HashSet<String>() {{
        add("妙用俏生活");
    }};

    private final String SINGLE_VIDEO_UID = "76862180";

    private final Integer VIDEO_NUM = 3;

    @Autowired
    private MatchVideoMapper matchVideoMapper;

    @Autowired
    private CrawlerVideoMapper crawlerVideoMapper;

    @Autowired
    private PublicContentServiceImpl publicContentService;

    @Autowired
    private AigcService aigcService;

    @Autowired
    private SortService sortService;

    @Autowired
    private KimiService kimiService;

    @Autowired
    private VideoService videoService;

    @Autowired
    private SingleVideoSourceMapper singleVideoSourceMapper;

    @Autowired
    private PublishMatchContentMapper publishMatchContentMapper;

    @Autowired
    private VideoEndScreenMapper videoEndScreenMapper;

    @Autowired
    private AIVideoAbTestMapper aiVideoAbTestMapper;

    @Autowired
    private PiaoquanSourceVideoPoolMapper piaoquanSourceVideoPoolMapper;

    @Autowired
    private VideoKeywordsMapMapper videoKeywordsMapMapper;

    @Autowired
    private ContentPlatformVideoMapper contentPlatformVideoMapper;

    @Autowired
    private MatchService matchService;

    public void addMatchContent(MatchVideo matchVideo, String publishContentId) {
        PublishMatchContent publishMatchContent = new PublishMatchContent();
        publishMatchContent.setTraceId(matchVideo.getTraceId());
        publishMatchContent.setPublishContentId(publishContentId);
        publishMatchContent.setContentId(matchVideo.getContentId());
        publishMatchContent.setGhId(matchVideo.getGhId());
        publishMatchContent.setStatus(matchVideo.getContentStatus());
        publishMatchContentMapper.insertSelective(publishMatchContent);
    }

    public MatchVideo getContent(String contentId, String ghId, Integer publishFlag) {
        MatchVideoExample matchVideoExample = new MatchVideoExample();
        matchVideoExample.createCriteria().andGhIdEqualTo(ghId).andContentIdEqualTo(contentId)
                .andPublishFlagEqualTo(publishFlag);
        List<MatchVideo> matchVideos = matchVideoMapper.selectByExample(matchVideoExample);
        if (CollectionUtils.isEmpty(matchVideos)) {
            return null;
        }
        return matchVideos.get(0);
    }

    public MatchVideo getMatchVideo(PublishContent publishContent, PlanAccount planAccount) {
        MatchVideoExample matchVideoExample = new MatchVideoExample();
        matchVideoExample.createCriteria().andGhIdEqualTo(planAccount.getGhId()).andContentIdEqualTo(publishContent.getSourceId())
                .andPublishFlagIn(Arrays.asList(1, 2));
        matchVideoExample.setOrderByClause("publish_flag desc");
        List<MatchVideo> matchVideos = matchVideoMapper.selectByExample(matchVideoExample);
        if (!CollectionUtils.isEmpty(matchVideos)) {
            return matchVideos.get(0);
        }
        MatchVideo newMatchVideo = matchVideos.get(0);
        newMatchVideo.setTraceId(String.format("search-%s-%s", UUID.randomUUID(), System.currentTimeMillis() / 1000));
        newMatchVideo.setContentId(publishContent.getSourceId());
        newMatchVideo.setFlowPoolLevel(publishContent.getContentPoolType());
        newMatchVideo.setGhId(planAccount.getGhId());
        newMatchVideo.setAccountName(planAccount.getAccountName());
        newMatchVideo.setContentStatus(ContentStatusEnum.SUCCESS_3.getStatusCode());
        newMatchVideo.setPublishFlag(PublicFlagEnum.NOT_PUBLIC.getStatusCode());
        matchVideoMapper.insertSelective(newMatchVideo);
        return newMatchVideo;
    }

    public void updateMatchContent(PublishContent publishContent,
                                   List<PublishMiniprogram> publishMiniprogramList, MatchVideo matchVideo) {
        try {
            JSONArray jsonArray = getResponse(publishMiniprogramList);
            String traceId = matchVideo.getTraceId();
            MatchVideo updateMatchVideo = new MatchVideo();
            updateMatchVideo.setId(matchVideo.getId());
            updateMatchVideo.setContentStatus(ContentStatusEnum.SUCCESS_4.getStatusCode());
            updateMatchVideo.setPublishFlag(PublicFlagEnum.NOT_PUBLIC.getStatusCode());
            updateMatchVideo.setResponse(jsonArray.toJSONString());
            matchVideoMapper.updateByPrimaryKeySelective(updateMatchVideo);
            publicContentService.updatePublishContentTraceId(traceId, publishContent.getId());
        } catch (Exception e) {
            log.error("updateMatchContent error", e);
        }
    }

    private JSONArray getResponse(List<PublishMiniprogram> publishMiniprogramList) {
        JSONArray jsonArray = new JSONArray();
        for (PublishMiniprogram publishMiniprogram : publishMiniprogramList) {
            //生成match表json数据
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("uid", publishMiniprogram.getUserId());
            jsonObject.put("source", publishMiniprogram.getSource());
            jsonObject.put("kimiTitle", publishMiniprogram.getVideoTitle());
            jsonObject.put("videoId", publishMiniprogram.getVideoId());
            jsonObject.put("videoCover", publishMiniprogram.getVideoCover());
            jsonObject.put("videoPath", publishMiniprogram.getVideoPath());
            jsonObject.put("videoOss", publishMiniprogram.getVideoOssPath());
            jsonObject.put("rootSourceId", publishMiniprogram.getRootSourceId());
            jsonArray.add(jsonObject);
        }
        return jsonArray;
    }

    public List<PublishArticleData> getWaitingSort(PlanAccount planAccount) {
        //获取待排序数据
        GetContentsParam waitingSortParam = new GetContentsParam();
        waitingSortParam.setPlanId(planAccount.getPlanId());
        waitingSortParam.setAccountId(planAccount.getAccountId());
        waitingSortParam.setPageNum(1);
        waitingSortParam.setPageSize(planAccount.getPublishPreNum());
        LongArticleSystemContentVO longArticleSystemContentVO = aigcService.listWaitingSortContents(waitingSortParam);
        if (longArticleSystemContentVO == null
                || longArticleSystemContentVO.getTotalCount() < planAccount.getPublishPreMinNum()
                || CollectionUtils.isEmpty(longArticleSystemContentVO.getContentItemList())
                || longArticleSystemContentVO.getContentItemList().size() < planAccount.getPublishPreMinNum()) {
            return new ArrayList<>();
        }
        log.info("aigcService.listWaitingSortContents.total={}", longArticleSystemContentVO.getTotalCount());
        List<PublishArticleData> waitSortList = new ArrayList<>();
        for (ContentItemVO contentItemVO : longArticleSystemContentVO.getContentItemList()) {
            PublishArticleData publishArticleData = new PublishArticleData();
            publishArticleData.setId(contentItemVO.getPublishContentId());
            publishArticleData.setSourceId(contentItemVO.getSourceId());
            publishArticleData.setTitle(contentItemVO.getTitle());
            publishArticleData.setContentPoolType(contentItemVO.getFlowPoolLevelTag());
            publishArticleData.setSourceType(contentItemVO.getSourceType());
            waitSortList.add(publishArticleData);
        }
        return waitSortList;
    }

    public List<PublishArticleData> getSortList(PlanAccount planAccount) {
        ArticleSortRequest articleSortRequest = new ArticleSortRequest();
        articleSortRequest.setAccountId(planAccount.getAccountId());
        articleSortRequest.setAccountName(planAccount.getAccountName());
        articleSortRequest.setGhId(planAccount.getGhId());
        //测试参数
//            articleSortRequest.setStrategy("ArticleRankRandom");
//            articleSortRequest.setExcludeLog(true);
        articleSortRequest.setStrategy(planAccount.getSortStrategy());
        articleSortRequest.setPublishNum(planAccount.getPublishPreNum());
        if (Objects.equals(PublishGzhPushTypeEnum.auto_group_publish.getVal(), planAccount.getPushType())) {
            articleSortRequest.setPublishNum(planAccount.getPublishNum());
        }
        articleSortRequest.setPlanId(planAccount.getPlanId());
        articleSortRequest.setPushType(planAccount.getPushType());
        int excludeContentSum = 0;
        if (StringUtils.isNotEmpty(planAccount.getExcludeContentIndex())) {
            List<Integer> excludeContentIndexList = JSONArray.parseArray(planAccount.getExcludeContentIndex(), Integer.class);
            if (!CollectionUtils.isEmpty(excludeContentIndexList)) {
                articleSortRequest.setExcludeContentIndex(excludeContentIndexList);
                excludeContentSum = excludeContentIndexList.size();
            }
        }
        log.info("articleSortRequest={}", JSONObject.toJSONString(articleSortRequest));
        ArticleSortResponse articleSortResponse = sortService.publishArticleSort(articleSortRequest);

        if (articleSortResponse == null || articleSortResponse.getData() == null) {
            log.info("articleSortResponse={}", articleSortResponse);
            String message = String.format("排序结果为空 planId=%s accountName=%s", planAccount.getPlanId(), planAccount.getAccountName());
            LarkRobotUtil.sendMessage(message);

            return new ArrayList<>();
        }
        ArticleSortResponseData data = articleSortResponse.getData();
        if (!CollectionUtils.isEmpty(data.getFilter_list())) {
            List<FilterSortContentParam> list = new ArrayList<>();
            for (PublishArticleData publishArticleData : data.getFilter_list()) {
                FilterSortContentParam param = new FilterSortContentParam();
                param.setFilterReason(publishArticleData.getFilterReason());
                param.setPublishContentId(publishArticleData.getId());
                list.add(param);
            }
            FilterSortParam param = new FilterSortParam();
            param.setFilterSortContentList(list);
            aigcService.filterSortContents(param);
        }
        if (CollectionUtils.isEmpty(articleSortResponse.getData().getRank_list())
                || articleSortResponse.getData().getRank_list().size() < (planAccount.getPublishPreMinNum() - excludeContentSum)) {
            log.info("rank_list = {}", articleSortResponse.getData().getRank_list());
            String message = String.format("排序结果不足 planId=%s accountName=%s num=%s",
                    planAccount.getPlanId(), planAccount.getAccountName(), articleSortResponse.getData().getRank_list().size());
            LarkRobotUtil.sendMessage(message);
            return new ArrayList<>();
        }
        return data.getRank_list();
    }

    public List<VideoDetail> getPublishVideoDetail(MatchVideo matchVideo, Integer publicFlag, PlanAccount planAccount, PublishContent publishContent) {
        List<CrawlerVideo> contentMiniVideos = getContentMiniVideo(matchVideo.getContentId(), publicFlag, planAccount);
        log.info("getPublishVideoDetail contentMiniVideos={}", contentMiniVideos);
        if (CollectionUtils.isEmpty(contentMiniVideos)) {
            LarkRobotUtil.sendMessage("文章获取小程序为空 matchVideo.getContentId=" + matchVideo.getContentId()
                    + " planId=" + planAccount.getPlanId() + " accountName=" + planAccount.getAccountName());
            MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
            statusParam.setStatus(MatchResultStatusEnum.FAIL.getStatusCode());
            statusParam.setPublishContentId(publishContent.getPublishContentId());
            statusParam.setErrorMsg("小程序为空");
            aigcService.updateMatchMiniprogramStatus(statusParam);
            log.info("文章获取小程序为空 matchVideo={} planAccount={}", matchVideo, planAccount);
            matchService.notHasMiniVideos(publishContent.getSourceId());
            return new ArrayList<>();
        }
        List<VideoDetail> videoDetails = publishVideo(contentMiniVideos, matchVideo, publishContent);
        log.info("getPublishVideoDetail videoDetails={}", videoDetails);
        if (CollectionUtils.isEmpty(videoDetails)) {
            LarkRobotUtil.sendMessage("发布后小程序为空 matchVideo.getContentId=" + matchVideo.getContentId());
            return new ArrayList<>();
        }
        return videoDetails;
    }

    public List<VideoDetail> getOnlyMiniPublishVideoDetail(PublishContent publishContent) {
        List<VideoDetail> res = new ArrayList<>();
        SingleVideoSourceExample example = new SingleVideoSourceExample();
        example.createCriteria().andContentTraceIdEqualTo(publishContent.getSourceId()).andAuditStatusEqualTo(1);
        List<SingleVideoSource> singleVideoSources = singleVideoSourceMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(singleVideoSources)) {
            return null;
        }
        SingleVideoSource singleVideoSource = singleVideoSources.get(0);
        String ossPath = singleVideoSource.getVideoOssPath();
        String title = singleVideoSource.getMiniProgramTitle();
        //是否存在片尾引导
        int transformationType;
        int publishType = 0;
        String videoOriginType = "video_pool";
        Long videoOriginId = (long) singleVideoSource.getId();
        VideoEndScreen videoEndScreen = existVideoEndScreen(videoOriginId, videoOriginType);
        //查询到改写的视频  随机选择原视频或者改写后的视频
        if (videoEndScreen != null) {
            transformationType = TransformationEnum.EXIST.getType();
            //存在片尾引导后  50%概率走有片尾引导的视频
            publishType = ToolUtil.getRandom(2);
            if (publishType == 1) {
                ossPath = videoEndScreen.getNewOssPath();
                title = videoEndScreen.getTitle();
            }
        } else {
            transformationType = TransformationEnum.NOT_EXIST.getType();
        }
        VideoDetail videoDetail = videoService.publish(ossPath,
                SINGLE_VIDEO_UID, title);
        if (videoDetail == null) {
            return null;
        }
        logEvent(videoOriginType, videoOriginId, videoDetail.getVideoId(), transformationType, publishType);
        videoDetail.setCrawlerVideoId(singleVideoSource.getId());
        videoDetail.setKimiTitle(title);
        videoDetail.setUid(SINGLE_VIDEO_UID);
        videoDetail.setVideoOss(ossPath);
        String traceId = "direct-" + UUID.randomUUID() + "-" + System.currentTimeMillis() / 1000;
        videoDetail.setTraceId(traceId);
        res.add(videoDetail);
        return res;
    }

    public List<VideoDetail> publishVideo(List<CrawlerVideo> contentMiniVideos, MatchVideo matchVideo, PublishContent publishContent) {
        List<VideoDetail> videoDetails = new ArrayList<>();
        //ai头条视频改写实验
        if (ghIdSet.contains(matchVideo.getGhId()) && publishContent != null && Objects.equals(publishContent.getContentPoolType(), "autoArticlePoolLevel1")) {
            String sourceId = publishContent.getSourceId();
            AIVideoAbTestExample example = new AIVideoAbTestExample();
            example.createCriteria().andContentIdEqualTo(sourceId).andStatusEqualTo(1);
            List<AIVideoAbTest> aiVideoAbTests = aiVideoAbTestMapper.selectByExample(example);
            if (!CollectionUtils.isEmpty(aiVideoAbTests)) {
                LongArticlesText kimiText = kimiService.getKimiText(sourceId);
                String title = kimiText.getKimiTitle();
                List<AIVideoAbTest> abTestList = new ArrayList<>();
                AIVideoAbTest aiVideoAbTest = aiVideoAbTests.get(0);
                abTestList.add(aiVideoAbTest);
                int size = abTestList.size();
                for (int i = size; i < VIDEO_NUM; i++) {
                    AIVideoAbTest newAiVideoAbTest = new AIVideoAbTest();
                    BeanUtils.copyProperties(aiVideoAbTest, newAiVideoAbTest);
                    abTestList.add(newAiVideoAbTest);
                }
                for (int i = 0; i < abTestList.size(); i++) {
                    String videoCover;
                    CrawlerVideo crawlerVideo = contentMiniVideos.get(i);
                    PublishMiniprogram publishMiniprograms = publicContentService.getPublishMiniprograms(crawlerVideo.getId());
                    if (publishMiniprograms != null) {
                        videoCover = publishMiniprograms.getVideoCover();
                    } else {
                        VideoDetail videoDetail = videoService.publish(crawlerVideo.getVideoOssPath(), crawlerVideo.getUserId(), title);
                        videoCover = videoDetail.getCoverImgPath();
                    }
                    AIVideoAbTest aiVideoAbTest1 = abTestList.get(i);
                    VideoDetail videoDetail = videoService.publish(aiVideoAbTest1.getVideoOssPath(), SINGLE_VIDEO_UID, title);
                    videoDetail.setCrawlerVideoId(aiVideoAbTest1.getId());
                    videoDetail.setKimiTitle(title);
                    videoDetail.setUid(SINGLE_VIDEO_UID);
                    videoDetail.setVideoOss(aiVideoAbTest1.getVideoOssPath());
                    videoDetail.setSource("ai");
                    videoDetail.setTraceId(matchVideo.getTraceId());
                    videoDetail.setCoverImgPath(videoCover);
                    videoDetails.add(videoDetail);
                }
                aiLogEvent(1, matchVideo.getGhId(), sourceId, DateUtil.getThatDayDateString());
                return videoDetails;
            } else {
                aiLogEvent(0, matchVideo.getGhId(), sourceId, DateUtil.getThatDayDateString());
            }
        }
        //发布小程序
        int i = 0;
        for (CrawlerVideo crawlerVideo : contentMiniVideos) {
            i++;
            String videoPath = "";
            Integer crawlerVideoId = crawlerVideo.getId();
            String videoOssPath = crawlerVideo.getVideoOssPath();
            String platform = crawlerVideo.getPlatform();
            String userId = crawlerVideo.getUserId();
            String traceId = matchVideo.getTraceId();
            LongArticlesText kimiText = kimiService.getKimiText(crawlerVideo.getContentId());
            if (kimiText == null || StringUtils.isEmpty(kimiText.getKimiTitle())) {
                log.error("crawlerVideo kimiText is null contentId={}", crawlerVideo.getContentId());
                continue;
            }
            String title = kimiText.getKimiTitle();
            //更换目标公众号的文章的第二条视频
            if (accountNameSet.contains(matchVideo.getAccountName()) && i == 2) {
                Long videoId = null;
                int matchType = 0;
                ContentPlatformVideoExample contentPlatformVideoExample = new ContentPlatformVideoExample();
                contentPlatformVideoExample.setOrderByClause("dt desc, score desc");
                contentPlatformVideoExample.setPage(new Page<>(1, 1));
                List<ContentPlatformVideo> contentPlatformVideoList = contentPlatformVideoMapper.selectByExample(contentPlatformVideoExample);
                if (!CollectionUtils.isEmpty(contentPlatformVideoList)) {
                    videoId = contentPlatformVideoList.get(0).getVideoId();
                }
                String articleTitle = kimiText.getArticleTitle();
                VideoKeywordsMapExample videoKeywordsMapExample = new VideoKeywordsMapExample();
                videoKeywordsMapExample.createCriteria().andStatusEqualTo(1);
                List<VideoKeywordsMap> videoKeywordsMaps = videoKeywordsMapMapper.selectByExampleWithBLOBs(videoKeywordsMapExample);
                List<String> resultList = new ArrayList<>();
                List<String> matchKeys = new ArrayList<>();
                if (!CollectionUtils.isEmpty(videoKeywordsMaps)) {
                    String keywordsMap = videoKeywordsMaps.get(0).getKeywordsMap();
                    JiebaSegmenter js = new JiebaSegmenter();
                    resultList = js.sentenceProcess(articleTitle);
                    if (StringUtils.isNotEmpty(keywordsMap) && !CollectionUtils.isEmpty(resultList)) {
                        JSONObject jsonObject = JSONObject.parseObject(keywordsMap);
                        JSONArray result = new JSONArray();
                        for (String keyWord : resultList) {
                            JSONArray jsonArray = jsonObject.getJSONArray(keyWord);
                            if (!CollectionUtils.isEmpty(jsonArray)) {
                                matchKeys.add(keyWord);
                                result.addAll(jsonArray);
                            }
                        }
                        if (!CollectionUtils.isEmpty(result)) {
                            List<Long> videoList = result.toJavaList(Long.class);
                            ContentPlatformVideoExample example = new ContentPlatformVideoExample();
                            example.createCriteria().andVideoIdIn(videoList);
                            example.setOrderByClause("score desc");
                            example.setPage(new Page<>(1, 1));
                            List<ContentPlatformVideo> contentPlatformVideos = contentPlatformVideoMapper.selectByExample(example);
                            if (!CollectionUtils.isEmpty(contentPlatformVideos)) {
                                matchType = 1;
                                videoId = contentPlatformVideos.get(0).getVideoId();
                            }
                        }
                    }
                }
                if (videoId != null) {
                    JSONObject videoDetail = VideoUtils.getVideoDetail(videoId);
                    if (videoDetail != null) {
                        crawlerVideoId = Math.toIntExact(videoId);
                        videoOssPath = videoDetail.getString("ossVideoPath");
                        videoPath = videoDetail.getString("videoPath");
                        JSONObject logJson = new JSONObject();
                        logJson.put("content_id", kimiText.getContentId());
                        logJson.put("article_title", kimiText.getArticleTitle());
                        logJson.put("match_video_id", videoId);
                        logJson.put("match_type", matchType);
                        logJson.put("title_keys", resultList);
                        logJson.put("match_key", matchKeys);
                        log.info(logJson.toJSONString());
                        log.info("replace video new videoId={}", videoId);
                    }
                }
            }
            if (StringUtils.isEmpty(userId)) {
                userId = SINGLE_VIDEO_UID;
            }
            VideoDetail videoDetail = videoService.publish(videoOssPath, userId, title);
            if (videoDetail == null) {
                continue;
            }
            if (StringUtils.isEmpty(videoDetail.getVideoPath())) {
                videoDetail.setVideoPath(videoPath);
            }
            videoDetail.setCrawlerVideoId(crawlerVideoId);
            videoDetail.setKimiTitle(title);
            videoDetail.setUid(userId);
            videoDetail.setVideoOss(videoOssPath);
            videoDetail.setSource(platform);
            videoDetail.setTraceId(traceId);
            videoDetails.add(videoDetail);
            if (StringUtils.isNotEmpty(crawlerVideo.getCoverOssPath())) {
                videoDetail.setCoverOss(crawlerVideo.getCoverOssPath());
                String rawImgPath = "https://rescdn.yishihui.com/" + crawlerVideo.getCoverOssPath()
                        + "?x-oss-process=image/resize,m_fill,w_600,h_480,limit_0/format,jpg/watermark,image_eXNoL3BpYy93YXRlcm1hcmtlci9pY29uX3BsYXlfd2hpdGUucG5nP3gtb3NzLXByb2Nlc3M9aW1hZ2UvcmVzaXplLHdfMTQ0,g_center";
                videoDetail.setRawImgPath(rawImgPath);
            }
        }
        return videoDetails;
    }


    public List<CrawlerVideo> getContentMiniVideo(String contentId, Integer publicFlag, PlanAccount planAccount) {
        try {
            CrawlerVideoExample example = new CrawlerVideoExample();
            example.setOrderByClause("score desc");
            CrawlerVideoExample.Criteria criteria = example.createCriteria().andDownloadStatusEqualTo(2)
                    .andContentIdEqualTo(contentId);
            //不发布需要审核后才能使用
            if (PublicFlagEnum.NOT_PUBLIC.getStatusCode() == publicFlag) {
                criteria.andStatusNotEqualTo(2).andIsIllegalEqualTo(0);
            }
            List<CrawlerVideo> crawlerVideoList = crawlerVideoMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(crawlerVideoList)) {
                return null;
            }
            if (planAccount == null) {
                return crawlerVideoList;
            }
            if (crawlerVideoList.size() < VIDEO_NUM) {
                CrawlerVideo crawlerVideo = crawlerVideoList.get(0);
                int needAddCount = VIDEO_NUM - crawlerVideoList.size();
                for (int i = 0; i < needAddCount; i++) {
                    CrawlerVideo copyCrawlerVideo = new CrawlerVideo();
                    BeanUtils.copyProperties(crawlerVideo, copyCrawlerVideo);
                    crawlerVideoList.add(copyCrawlerVideo);
                }
            }
            if (crawlerVideoList.size() > VIDEO_NUM) {
                return crawlerVideoList.subList(0, VIDEO_NUM);
            }
            log.info("getContentMiniVideo crawlerVideoList={}", crawlerVideoList);
            return crawlerVideoList;
        } catch (Exception e) {
            log.error("getContentMiniVideo error", e);
        }
        return new ArrayList<>();
    }

    public VideoEndScreen existVideoEndScreen(Long id, String type) {
        VideoEndScreenExample example = new VideoEndScreenExample();
        example.createCriteria().andVideoOriginIdEqualTo(id).andVideoOriginTypeEqualTo(type).andStatusEqualTo(2);
        List<VideoEndScreen> videoEndScreens = videoEndScreenMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(videoEndScreens)) {
            return null;
        }
        return videoEndScreens.get(0);
    }

    public void logEvent(String videoOriginType, Long videoOriginId, String videoId,
                         int transformationType, int publishType) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("code", "changwen_video_publish");
        JSONObject data = new JSONObject();
        data.put("video_origin_type", videoOriginType);
        data.put("video_origin_id", videoOriginId);
        data.put("video_id", videoId);
        data.put("transformation_type", transformationType);
        if (transformationType == 1) {
            data.put("publish_type", publishType);
        }
        jsonObject.put("data", data);
        log.info(jsonObject.toJSONString());
    }

    public void aiLogEvent(Integer testStatus, String ghId, String contentId, String publishDate) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("code", "ad_video_ab_test");
        jsonObject.put("testStatus", testStatus);
        jsonObject.put("gh_id", ghId);
        jsonObject.put("content_id", contentId);
        jsonObject.put("publish_date", publishDate);
        log.info(jsonObject.toJSONString());
    }
}
