package com.tzld.piaoquan.longarticle.service.local.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.longarticle.common.constants.TimeConstant;
import com.tzld.piaoquan.longarticle.common.enums.*;
import com.tzld.piaoquan.longarticle.dao.mapper.crawler.ArticleUseGroupMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.crawler.FwhDailyPublishDetailMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.*;
import com.tzld.piaoquan.longarticle.model.bo.MatchContent;
import com.tzld.piaoquan.longarticle.model.bo.VideoDetail;
import com.tzld.piaoquan.longarticle.model.dto.AuditContentRequest;
import com.tzld.piaoquan.longarticle.model.dto.MiniprogramCardRequest;
import com.tzld.piaoquan.longarticle.model.dto.PublishArticleData;
import com.tzld.piaoquan.longarticle.model.po.crawler.ArticleUseGroup;
import com.tzld.piaoquan.longarticle.model.po.crawler.ArticleUseGroupExample;
import com.tzld.piaoquan.longarticle.model.po.crawler.FwhDailyPublishDetail;
import com.tzld.piaoquan.longarticle.model.po.longarticle.*;
import com.tzld.piaoquan.longarticle.model.vo.*;
import com.tzld.piaoquan.longarticle.service.local.AccountService;
import com.tzld.piaoquan.longarticle.service.local.CoreService;
import com.tzld.piaoquan.longarticle.service.local.PlanAccountService;
import com.tzld.piaoquan.longarticle.service.remote.AigcService;
import com.tzld.piaoquan.longarticle.service.remote.MatchService;
import com.tzld.piaoquan.longarticle.service.remote.VideoService;
import com.tzld.piaoquan.longarticle.service.remote.impl.SortServiceImpl;
import com.tzld.piaoquan.longarticle.utils.DateUtil;
import com.tzld.piaoquan.longarticle.utils.LarkRobotUtil;
import com.tzld.piaoquan.longarticle.utils.TimeZoneUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.*;
import java.util.concurrent.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
public class CoreServiceImpl implements CoreService {

    @Autowired
    private AigcService aigcService;

    @Autowired
    private ContentServiceImpl contentService;

    @Autowired
    private MatchService matchService;

    @Autowired
    private VideoService videoService;

    @Autowired
    private CardServiceImpl cardService;

    @Autowired
    private PlanAccountMapper planAccountMapper;

    @Autowired
    private PlanAccountService planAccountService;

    @Autowired
    private PublishMiniprogramMapper publishMiniprogramMapper;

    @Autowired
    private PublishContentMapper publishContentMapper;

    @Autowired
    private PublicContentServiceImpl publicContentService;

    @Autowired
    private RootSourceServiceImpl rootSourceService;

    @Autowired
    private AccountService accountService;

    @Autowired
    private ArticleUseGroupMapper articleUseGroupMapper;

    @Autowired
    private GroupSendResultMapper groupSendResultMapper;

    @Autowired
    private GroupSendOpenIdMapper groupSendOpenIdMapper;

    @Autowired
    private MatchVideoServiceImpl matchVideoService;

    @Autowired
    private FwhDailyPublishDetailMapper fwhDailyPublishDetailMapper;

    @ApolloJsonValue("${fwhGhIdList:[]}")
    private List<String> fwhGhIdList;

    public void initAccountSpecialSettings() {
        List<SpecialSettingVO> specialSettings = aigcService.getSpecialSetting();
        if (CollectionUtils.isEmpty(specialSettings)) {
            return;
        }
        for (SpecialSettingVO specialSettingVO : specialSettings) {
            List<AccountSpecialSettings> existAccountSpecialSettings =
                    accountService.getAccountSpecialSettingsById(specialSettingVO.getId());
            if (!CollectionUtils.isEmpty(existAccountSpecialSettings)) {
                if (!CollectionUtils.isEmpty(specialSettingVO.getAccountList())) {
                    List<String> collect = specialSettingVO.getAccountList().stream().map(SpecialSettingAccountVO::getId)
                            .collect(Collectors.toList());
                    for (AccountSpecialSettings accountSpecialSettings : existAccountSpecialSettings) {
                        if (!collect.contains(accountSpecialSettings.getAccountId())) {
                            accountService.delAccountSpecialSettings(accountSpecialSettings.getId());
                        }
                    }
                } else {
                    for (AccountSpecialSettings accountSpecialSettings : existAccountSpecialSettings) {
                        accountService.delAccountSpecialSettings(accountSpecialSettings.getId());
                    }
                }
            }
            if (CollectionUtils.isEmpty(specialSettingVO.getAccountList())) {
                continue;
            }
            for (SpecialSettingAccountVO specialSettingAccountVO : specialSettingVO.getAccountList()) {
                AccountSpecialSettings accountSpecialSettings = new AccountSpecialSettings();
                accountSpecialSettings.setSpecialSettingId(specialSettingVO.getId());
                accountSpecialSettings.setEffectiveDate(specialSettingVO.getEffectiveDate());
                accountSpecialSettings.setAccountId(specialSettingAccountVO.getId());
                accountSpecialSettings.setPublishPushTime(specialSettingVO.getPushTime());
                accountSpecialSettings.setPushType(specialSettingVO.getPushType());
                accountSpecialSettings.setPublishNum(specialSettingVO.getPushNum());
                accountSpecialSettings.setOpenStatus(specialSettingVO.getOpenStatus());
                if (!CollectionUtils.isEmpty(specialSettingVO.getExcludeArticleIndexList())) {
                    List<Integer> collect = specialSettingVO.getExcludeArticleIndexList().stream()
                            .map(Integer::parseInt).collect(Collectors.toList());
                    accountSpecialSettings.setExcludeContentIndex(JSON.toJSONString(collect));
                }
                accountService.saveOrUpdateAccountSpecialSettings(accountSpecialSettings);
            }
        }

    }

    @Override
    public void initPlanAccount() {
        log.info("initPlanAccount start");
        //例外设置查询和存储
        //获取当天不能发送操作计划的账号列表
        List<AccountSpecialSettings> accountSpecialSettings = accountService.getAccountSpecialSettings();
        List<String> specialAccountIds = accountSpecialSettings.stream().map(AccountSpecialSettings::getAccountId).collect(Collectors.toList());
        //获取当天需要额外推送的例外设置账号列表
        List<AccountSpecialSettings> sendAccountSpecialSettings = accountService.getSendAccountSpecialSettings();
        //操作计划遍历
        List<Plan> planList = planAccountService.getPlanList();
        Set<String> planIdSet = planList.stream().map(Plan::getPlanId).collect(Collectors.toSet());
        List<LongArticleSystemPlan> allLongArticleSystemPlans = aigcService.getAllLongArticleSystemPlan();
        for (LongArticleSystemPlan longArticleSystemPlan : allLongArticleSystemPlans) {
            Plan plan = createPlan(longArticleSystemPlan);
            planAccountService.saveOrUpdatePlan(plan);
            planIdSet.remove(longArticleSystemPlan.getId());
            //获取排序策略
            Map<String, String> sortStgMap = new HashMap<>();
            List<GzhArticleSortTaskParam> gzhArticleSortTask = longArticleSystemPlan.getGzhArticleSortTask();
            for (GzhArticleSortTaskParam gzhArticleSortTaskParam : gzhArticleSortTask) {
                String strategy = gzhArticleSortTaskParam.getStrategy();
                List<String> accountIds = gzhArticleSortTaskParam.getAccountIds();
                for (String accountId : accountIds) {
                    sortStgMap.put(accountId, strategy);
                }
            }
            //获取小程序策略
            Map<String, MiniprogramInsertTaskParam> miniStgMap = new HashMap<>();
            List<MiniprogramInsertTaskParam> miniprogramInsertTasks = longArticleSystemPlan.getMiniprogramInsertTasks();
            for (MiniprogramInsertTaskParam miniprogramInsertTaskParam : miniprogramInsertTasks) {
                List<String> accountIds = miniprogramInsertTaskParam.getAccountIds();
                for (String accountId : accountIds) {
                    miniStgMap.put(accountId, miniprogramInsertTaskParam);
                }
            }
            List<PlanAccount> planAccounts = planAccountService.getPlanAccountByPlanId(longArticleSystemPlan.getId());
            Set<String> accountIdSet = planAccounts.stream().map(PlanAccount::getAccountId).collect(Collectors.toSet());
            for (LongArticleSystemPlanAccount account : longArticleSystemPlan.getAccountList()) {
                PlanAccount planAccount = new PlanAccount();
                String accountId = account.getId();
                accountIdSet.remove(accountId);
                planAccount.setPlanId(longArticleSystemPlan.getId());
                planAccount.setPlanName(longArticleSystemPlan.getName());
                planAccount.setAccountId(accountId);
                planAccount.setAccountName(account.getName());
                planAccount.setGhId(account.getGhId());
                planAccount.setPublishOpenFlag(account.getPublishOpenFlag());
                planAccount.setPublishRate(longArticleSystemPlan.getPublishRate());
                planAccount.setPublishDate(longArticleSystemPlan.getPublishDate());
                planAccount.setPublishNum(longArticleSystemPlan.getPublishNum());
                planAccount.setPublishWindowStart(longArticleSystemPlan.getPublishWindowStart());
                planAccount.setPublishWindowEnd(longArticleSystemPlan.getPublishWindowEnd());
                planAccount.setPublishTimeInterval(longArticleSystemPlan.getPublishTimeInterval());
                planAccount.setPublishNum(longArticleSystemPlan.getPublishNum());
                planAccount.setPublishPreNum(longArticleSystemPlan.getPublishPerNum());
                planAccount.setPublishPreMinNum(longArticleSystemPlan.getPublishPerMinNum());
                planAccount.setPublishPushTime(longArticleSystemPlan.getPublishPushTime());
                planAccount.setPushType(longArticleSystemPlan.getPushType());
                planAccount.setMiniprogramInsertFlag(longArticleSystemPlan.getMiniprogramInsertFlag());
                MiniprogramInsertTaskParam miniprogramInsertTaskParam = miniStgMap.get(accountId);
                if (miniprogramInsertTaskParam != null) {
                    planAccount.setMiniprogramUseType(miniprogramInsertTaskParam.getMiniprogramUseType());
                    planAccount.setMiniStrategy(miniprogramInsertTaskParam.getStrategy());
                }
                if (sortStgMap.containsKey(accountId)) {
                    planAccount.setSortStrategy(sortStgMap.get(accountId));
                }
                planAccount.setPushType(longArticleSystemPlan.getPushType());
                if (!CollectionUtils.isEmpty(specialAccountIds) && specialAccountIds.contains(planAccount.getAccountId())) {
                    planAccount.setSpecialNotSend(SpecialNotSendEnum.NOT_SEND.getStatusCode());
                }
                planAccountService.saveOrUpdatePlanAccount(planAccount);
            }
            if (CollectionUtils.isEmpty(accountIdSet)) {
                continue;
            }
            for (String accountId : accountIdSet) {
                planAccountService.delAccount(longArticleSystemPlan.getId(), accountId);
            }

        }
        log.info("initPlanAccount end");
        if (!CollectionUtils.isEmpty(planIdSet)) {
            for (String planId : planIdSet) {
                planAccountService.delPlan(planId);
            }
        }

        //例外设置处理
        setSpecialAccount(accountSpecialSettings, sendAccountSpecialSettings);

    }

    public void setSpecialAccount(List<AccountSpecialSettings> accountSpecialSettings,
                                  List<AccountSpecialSettings> sendAccountSpecialSettings) {

        //当天计划不发送
        log.info("accountSpecialSettings={}", accountSpecialSettings);
        log.info("sendAccountSpecialSettings={}", sendAccountSpecialSettings);
        for (AccountSpecialSettings accountSpecialSetting : accountSpecialSettings) {
            List<PlanAccount> planAccountList = planAccountService.getPlanAccount(accountSpecialSetting.getAccountId());
            if (CollectionUtils.isEmpty(planAccountList)) {
                continue;
            }
            for (PlanAccount planAccount : planAccountList) {
                planAccount.setSpecialNotSend(SpecialNotSendEnum.NOT_SEND.getStatusCode());
                planAccountService.saveOrUpdatePlanAccount(planAccount);
            }
        }

        //额外发送
        for (AccountSpecialSettings accountSpecialSetting : sendAccountSpecialSettings) {
            List<PlanAccount> specialPlanAccount = planAccountService.getSpecialPlanAccount(accountSpecialSetting.getAccountId());
            if (CollectionUtils.isEmpty(specialPlanAccount)) {
                List<PlanAccount> planAccountList = planAccountService.getNormalPlanAccount(accountSpecialSetting.getAccountId());
                if (CollectionUtils.isEmpty(planAccountList)) {
                    continue;
                }
                Optional<PlanAccount> firstAccount = planAccountList.stream().min((a1, a2) -> -a1.getPublishNum().compareTo(a2.getPublishNum()));
                PlanAccount targetPlanAccount = firstAccount.get();
                updateAccountSetting(accountSpecialSetting, targetPlanAccount);
                targetPlanAccount.setStatus(0);
                targetPlanAccount.setIsSpecialSetting(1);
                targetPlanAccount.setSpecialNotSend(SpecialNotSendEnum.SEND.getStatusCode());
                targetPlanAccount.setId(null);
                targetPlanAccount.setCreateTime(new Date());
                targetPlanAccount.setUpdateTime(new Date());
                planAccountService.addPlanAccount(targetPlanAccount);
            } else {
                PlanAccount planAccount = specialPlanAccount.get(0);
                updateAccountSetting(accountSpecialSetting, planAccount);
                planAccountService.updatePlanAccount(planAccount);
            }
        }
    }

    private void updateAccountSetting(AccountSpecialSettings accountSpecialSettings, PlanAccount planAccount) {
        planAccount.setPushType(accountSpecialSettings.getPushType());
        if (PublishGzhPushTypeEnum.manual_push.getVal().equals(accountSpecialSettings.getPushType())) {
            planAccount.setPublishWindowStart(accountSpecialSettings.getPublishPushTime());
            //保证手动推送的结束时间在推送开始时间之后
            planAccount.setPublishWindowEnd("23:00:00");
        }
        planAccount.setPublishPushTime(accountSpecialSettings.getPublishPushTime());
        planAccount.setPublishPreNum(accountSpecialSettings.getPublishNum());
        planAccount.setPublishNum(accountSpecialSettings.getPublishNum());
        int size = 0;
        if (StringUtils.isNotEmpty(accountSpecialSettings.getExcludeContentIndex())) {
            size = JSONArray.parseArray(accountSpecialSettings.getExcludeContentIndex(), Integer.class).size();
        }
        if (planAccount.getPublishPreMinNum() > accountSpecialSettings.getPublishNum() - size) {
            planAccount.setPublishPreMinNum(accountSpecialSettings.getPublishNum() - size);
        }
        planAccount.setExcludeContentIndex(accountSpecialSettings.getExcludeContentIndex());
    }


    private Plan createPlan(LongArticleSystemPlan longArticleSystemPlan) {
        Plan plan = new Plan();
        plan.setPlanId(longArticleSystemPlan.getId());
        plan.setPlanName(longArticleSystemPlan.getName());
        plan.setPublishRate(longArticleSystemPlan.getPublishRate());
        plan.setPublishDate(longArticleSystemPlan.getPublishDate());
        plan.setPublishWindowStart(longArticleSystemPlan.getPublishWindowStart());
        plan.setPublishWindowEnd(longArticleSystemPlan.getPublishWindowEnd());
        plan.setPublishTimeInterval(longArticleSystemPlan.getPublishTimeInterval());
        plan.setPublishNum(longArticleSystemPlan.getPublishNum());
        plan.setPublishPreNum(longArticleSystemPlan.getPublishPerNum());
        plan.setPublishPreMinNum(longArticleSystemPlan.getPublishPerMinNum());
        plan.setPublishPushTime(longArticleSystemPlan.getPublishPushTime());
        plan.setPushType(longArticleSystemPlan.getPushType());
        plan.setMiniprogramInsertFlag(longArticleSystemPlan.getMiniprogramInsertFlag());
        return plan;
    }


    // 定义一个阻塞队列
    private static final ArrayBlockingQueue<MatchContent> matchQueue = new ArrayBlockingQueue<>(1000000);

    private static final ArrayBlockingQueue<PlanAccount> coreQueue = new ArrayBlockingQueue<>(1000000);


    private static final int size = 10;
    // 定义一个线程池，设置消费线程的数量
    private static final ThreadPoolExecutor matchPoolExecutor = (ThreadPoolExecutor) Executors.newFixedThreadPool(size);

    private static final ThreadPoolExecutor corePoolExecutor = (ThreadPoolExecutor) Executors.newFixedThreadPool(5);


    @Override
    public void matchContent() throws InterruptedException {
        if (matchPoolExecutor.getCorePoolSize() - matchPoolExecutor.getActiveCount() > 0) {
            int threadSize = matchPoolExecutor.getCorePoolSize() - matchPoolExecutor.getActiveCount();
            log.info("threadNum={}", threadSize);
            CountDownLatch countDownLatch = new CountDownLatch(threadSize);
            // 启动消费者线程
            for (int i = 0; i < threadSize; i++) {
                matchPoolExecutor.submit(new Thread(() -> {
                    log.info("启动匹配小程序线程");
                    while (true) {
                        try {
                            // 超过 2 分钟没有数据，销毁当前线程
                            MatchContent matchContent = matchQueue.poll(2, TimeUnit.MINUTES); // 等待最多 2 分钟
                            if (matchContent == null) {
                                break; // 退出当前线程
                            }
                            //只匹配小程序文章不做处理
                            if (Objects.equals(matchContent.getSourceType(), SourceTypesEnum.longArticleVideoPoolSource.getVal())) {
                                MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                                statusParam.setStatus(MatchResultStatusEnum.SUCCESS.getStatusCode());
                                statusParam.setPublishContentId(matchContent.getPublishContentId());
                                aigcService.updateMatchMiniprogramStatus(statusParam);
                                continue;
                            }
                            MatchVideo matchVideo = contentService.getContent(matchContent.getSourceId(),
                                    matchContent.getGhId(), PublicFlagEnum.NOT_PUBLIC.getStatusCode());
                            if (matchVideo != null) {
                                if (ContentStatusEnum.isSuccess(matchVideo.getContentStatus())) {
                                    MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                                    statusParam.setStatus(MatchResultStatusEnum.SUCCESS.getStatusCode());
                                    statusParam.setPublishContentId(matchContent.getPublishContentId());
                                    aigcService.updateMatchMiniprogramStatus(statusParam);
                                    contentService.addMatchContent(matchVideo, matchContent.getPublishContentId());
                                    continue;
                                }
                                if (ContentStatusEnum.isFail(matchVideo.getContentStatus())) {
                                    MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                                    statusParam.setStatus(MatchResultStatusEnum.FAIL.getStatusCode());
                                    statusParam.setPublishContentId(matchContent.getPublishContentId());
                                    String errorMessage = ContentStatusEnum.getErrorMessage(matchVideo.getContentStatus());
                                    statusParam.setErrorMsg(errorMessage);
                                    aigcService.updateMatchMiniprogramStatus(statusParam);
                                    contentService.addMatchContent(matchVideo, matchContent.getPublishContentId());
                                }
                                if (matchVideoService.existMatchVideoResult(matchVideo)) {
                                    MatchVideo matchVideo1 = contentService.getContent(matchContent.getSourceId(),
                                            matchContent.getGhId(), PublicFlagEnum.NOT_PUBLIC.getStatusCode());
                                    if (ContentStatusEnum.isSuccess(matchVideo1.getContentStatus())) {
                                        MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                                        statusParam.setStatus(MatchResultStatusEnum.SUCCESS.getStatusCode());
                                        statusParam.setPublishContentId(matchContent.getPublishContentId());
                                        boolean res = aigcService.updateMatchMiniprogramStatus(statusParam);
                                        if (res) {
                                            contentService.addMatchContent(matchVideo, matchContent.getPublishContentId());
                                            AuditContentRequest auditContentRequest = new AuditContentRequest();
                                            auditContentRequest.setContentId(matchContent.getSourceId());
                                            auditContentRequest.setFlowPoolLevel(matchContent.getFlowPoolLevelTag());
                                            matchService.addAudit(auditContentRequest);
                                        }
                                        continue;
                                    }
                                    if (ContentStatusEnum.isFail(matchVideo1.getContentStatus())) {
                                        MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                                        statusParam.setStatus(MatchResultStatusEnum.FAIL.getStatusCode());
                                        statusParam.setPublishContentId(matchContent.getPublishContentId());
                                        String errorMessage = ContentStatusEnum.getErrorMessage(matchVideo1.getContentStatus());
                                        statusParam.setErrorMsg(errorMessage);
                                        aigcService.updateMatchMiniprogramStatus(statusParam);
                                        contentService.addMatchContent(matchVideo, matchContent.getPublishContentId());
                                    }
                                }
                            } else {
                                MiniprogramCardRequest request = new MiniprogramCardRequest();
                                request.setGhId(matchContent.getGhId());
                                request.setAccountName(matchContent.getAccountName());
                                request.setContent(matchContent.getContent());
                                request.setTitle(matchContent.getTitle());
                                //请求到新服务
                                request.setStrategy("strategy_v2");
                                request.setArticleId(matchContent.getSourceId());
                                request.setFlowPoolLevelTag(matchContent.getFlowPoolLevelTag());
                                request.setPublishFlag(PublicFlagEnum.NOT_PUBLIC.getStatusCode());
                                request.setVideoPoolTraceId(matchContent.getVideoPoolTraceId());
                                Integer status = matchService.matchMiniprogramVideo(request);
                                if (status == MatchRequestStatusEnum.NOT_CONTENT.getStatusCode()) {
                                    MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                                    statusParam.setStatus(MatchResultStatusEnum.FAIL.getStatusCode());
                                    statusParam.setPublishContentId(matchContent.getPublishContentId());
                                    String errorMessage = "没有文章内容";
                                    statusParam.setErrorMsg(errorMessage);
                                    aigcService.updateMatchMiniprogramStatus(statusParam);
                                }
                            }
                        } catch (InterruptedException e) {
                            log.error("匹配小程序线程异常", e);
                        }
                    }
                    log.info("启动匹配小程序线程结束");
                    countDownLatch.countDown();
                }));
            }

            List<PlanAccount> matchPlanAccounts = planAccountService.getMatchPlanAccount();
            if (CollectionUtils.isEmpty(matchPlanAccounts)) {
                return;
            }
            for (PlanAccount planAccount : matchPlanAccounts) {
                GetContentsParam param = new GetContentsParam();
                param.setAccountId(planAccount.getAccountId());
                param.setPlanId(planAccount.getPlanId());
                LongArticleSystemContentVO longArticleSystemContentVO = aigcService.getContentItemList(param);
                if (longArticleSystemContentVO == null || CollectionUtils.isEmpty(longArticleSystemContentVO.getContentItemList())) {
                    continue;
                }
                log.info("longArticleSystemContentVO totalCount={}", longArticleSystemContentVO.getTotalCount());
                for (ContentItemVO contentItemVO : longArticleSystemContentVO.getContentItemList()) {
                    MatchContent matchContent = new MatchContent();
                    matchContent.setSourceId(contentItemVO.getSourceId());
                    matchContent.setGhId(planAccount.getGhId());
                    matchContent.setPublishContentId(contentItemVO.getPublishContentId());
                    matchContent.setAccountName(planAccount.getAccountName());
                    matchContent.setContent(contentItemVO.getContent());
                    matchContent.setTitle(contentItemVO.getTitle());
                    matchContent.setFlowPoolLevelTag(contentItemVO.getFlowPoolLevelTag());
                    matchContent.setSourceType(contentItemVO.getSourceType());
                    matchContent.setChannelContentId(contentItemVO.getChannelContentId());
                    matchContent.setVideoPoolTraceId(contentItemVO.getVideoPoolTraceId());
                    try {
                        matchQueue.put(matchContent);
                    } catch (InterruptedException e) {
                        log.error("matchQueue put error", e);
                    }
                }
            }
            countDownLatch.await();
        }
    }


    private boolean effectiveTime(String startWindow, String endWindow, TimeZoneUtil.Timezone timezone) {
        if (!org.springframework.util.StringUtils.hasText(startWindow) || !org.springframework.util.StringUtils.hasText(endWindow)) {
            return true;
        }
        LocalTime currentTime = TimeZoneUtil
                .currentTime(Optional.ofNullable(timezone).orElse(TimeZoneUtil.Timezone.china));
        return currentTime.isAfter(LocalTime.parse(startWindow)) && currentTime.isBefore(LocalTime.parse(endWindow));
    }

    public boolean checkPlanAccount(PlanAccount planAccount) {
        boolean isGzhGroupPushPlan = Objects.nonNull(planAccount.getPushType())
                && PublishGzhPushTypeEnum.auto_group_publish.getVal().equals(planAccount.getPushType());
        boolean flag = true;
        // 固定日期
        if (planAccount.getPublishRate() == 1) {
            List<Integer> weeks = JSON.parseArray(planAccount.getPublishDate(), Integer.class);
            if (!weeks.contains(TimeZoneUtil.getTodayDayOfWeek(TimeZoneUtil.Timezone.china))) {
                planAccount.setStatus(2);
                planAccountMapper.updateByPrimaryKeySelective(planAccount);
                return false;
            }
        }
        // 指定时间
        if (isGzhGroupPushPlan && StringUtils.isNotEmpty(planAccount.getPublishPushTime())) {
            long pushTime = DateUtil.convertToTimestamp(planAccount.getPublishPushTime());
            long nowTime = new Date().getTime();
            // 当前时间距离目标时间超过30分钟
            if (nowTime + 30 * TimeConstant.MILLISECOND_MINUTE < pushTime) {
                return false;
            }
            // 当前时间已经超过目标时间600分钟
            if (nowTime - 600 * TimeConstant.MILLISECOND_MINUTE > pushTime) {
                return false;
            }
        } else if (StringUtils.isNotEmpty(planAccount.getPublishWindowStart())
                && StringUtils.isNotEmpty(planAccount.getPublishWindowEnd())) {
            if (!effectiveTime(planAccount.getPublishWindowStart(), planAccount.getPublishWindowEnd(),
                    TimeZoneUtil.Timezone.china)) {
                return false;
            }
            if (planAccount.getPublishTimeInterval() != null) {
                // 找到该账号最近发布的内容，判断时间间隔
                PublishContentExample example = new PublishContentExample();
                example.createCriteria().andStatusIn(Arrays.asList(1, 2))
                        .andCreateTimeGreaterThan(DateUtil.getThatDayDate())
                        .andPlanAccountIdEqualTo(planAccount.getId());
                example.setOrderByClause("create_time desc");
                List<PublishContent> publishContents = publishContentMapper.selectByExample(example);
                if (!CollectionUtils.isEmpty(publishContents)) {
                    long time = System.currentTimeMillis() - publishContents.get(0).getCreateTime().getTime();
                    if (time < planAccount.getPublishTimeInterval() * 60 * 1000L) {
                        return false;
                    }
                }
            }
        }
        PushRecordParam param = new PushRecordParam();
        param.setPlanId(planAccount.getPlanId());
        param.setPublishAccountId(planAccount.getAccountId());
        boolean sendSpecialSettings = accountService.isSendSpecialSettings(planAccount.getAccountId());

        //例外设置不查询aigc数量  一定会多发
        int aigcCount = 0;
        if (!sendSpecialSettings) {
            //查询aigc发布成功数量
            List<PushRecordVO> todayPushRecords = aigcService.getTodayPushRecords(param);
            if (!CollectionUtils.isEmpty(todayPushRecords)) {
                List<PushRecordVO> collect = todayPushRecords.stream()
                        .filter(e -> ((e.getPushStatus() != 3) && (e.getPublishStatus() != 3))).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(collect)) {
                    for (PushRecordVO pushRecordVO : collect) {
                        if (pushRecordVO.getPublishContentCount() != null) {
                            aigcCount += pushRecordVO.getPublishContentCount();
                        }
                    }
                }
            }
        }

        //长文系统发布数量
        int sendCount = publicContentService.getSendCount(planAccount.getId());
        //取发布数量多的
        sendCount = Math.max(aigcCount, sendCount);
        if (isGzhGroupPushPlan) {
            //公众号群发每天只能成功发布一次
            if (sendCount >= 1) {
                return false;
            }
            //公众号群发每天最多尝试3次
            if (planAccount.getRetryCount() >= 3) {
                planAccount.setStatus(3);
                planAccountMapper.updateByPrimaryKeySelective(planAccount);
                LarkRobotUtil.sendMessage("计划发布失败3次，请查看 账号计划id为" + planAccount.getPlanId());
                return false;
            }
        } else {
            //其他群发最多重试5次
            if (planAccount.getRetryCount() >= 5) {
                planAccount.setStatus(3);
                planAccountMapper.updateByPrimaryKeySelective(planAccount);
                LarkRobotUtil.sendMessage("计划发布失败5次，请查看 账号计划id为" + planAccount.getPlanId());
                return false;
            }
        }
        Integer publishNum = planAccount.getPublishNum();
        int size = 0;
        if (StringUtils.isNotEmpty(planAccount.getExcludeContentIndex())) {
            size = JSONArray.parseArray(planAccount.getExcludeContentIndex(), Integer.class).size();
        }
        if (publishNum == null || publishNum == 0) {
            LarkRobotUtil.sendMessage("发布数量设置为0 planAccountId=" + planAccount.getId());
            return false;
        }
        if (sendCount >= publishNum - size) {
            return false;
        }

        int todayNeedSendCount = publishNum - sendCount;
        // 本次推送数量
        Integer prePushNum = planAccount.getPublishPreNum();
        if (Objects.isNull(prePushNum)) {
            prePushNum = 1;
        }
        //如果预计推送数量大于剩余推送数量  本次数量设置为剩余推送数量
        if (prePushNum > todayNeedSendCount) {
            prePushNum = todayNeedSendCount;
        }
        // 自动发布消息，每次暂时限定发布一条
        if (PublishGzhPushTypeEnum.robopost.getVal().equals(planAccount.getPushType())) {
            prePushNum = 1;
        }
        // 本次最少推送数量
        Integer preMinPushNum = planAccount.getPublishPreMinNum();
        if (Objects.isNull(preMinPushNum)) {
            preMinPushNum = prePushNum;
        }
        if (preMinPushNum > prePushNum) {
            preMinPushNum = prePushNum;
        }
        planAccount.setPublishPreNum(prePushNum);
        planAccount.setPublishPreMinNum(preMinPushNum);
        return flag;
    }


    @Override
    public void core() throws InterruptedException {
        if (corePoolExecutor.getCorePoolSize() - corePoolExecutor.getActiveCount() > 0) {
            int threadSize = corePoolExecutor.getCorePoolSize() - corePoolExecutor.getActiveCount();
            log.info("threadNum={}", threadSize);
            CountDownLatch countDownLatch = new CountDownLatch(threadSize);
            // 启动消费者线程
            for (int i = 0; i < threadSize; i++) {
                corePoolExecutor.submit(new Thread(() -> {
                    log.info("启动核心操作线程");
                    while (true) {
                        try {
                            // 超过 2 分钟没有数据，销毁当前线程
                            PlanAccount planAccount = coreQueue.poll(2, TimeUnit.MINUTES); // 等待最多 2 分钟
                            if (planAccount == null) {
                                break; // 退出当前线程
                            }
                            operatePlanAccount(planAccount);
                        } catch (Exception e) {
                            log.error("", e);
                        }
                    }
                    log.info("核心操作线程结束");
                    countDownLatch.countDown();
                }));
            }
            List<PlanAccount> planAccounts;
            planAccounts = planAccountService.getSendNormalPlanAccount();
            if (CollectionUtils.isEmpty(planAccounts)) {
                return;
            }
            for (PlanAccount planAccount : planAccounts) {
                coreQueue.put(planAccount);
            }
            countDownLatch.await();
        }
    }

    public void operatePlanAccount(PlanAccount planAccount) {
        boolean flag = checkPlanAccount(planAccount);
        log.info("planAccount={}, flag={}", planAccount, flag);
        if (!flag) {
            return;
        }
        //获取待排序数据  数量不足会返回空list
        List<PublishArticleData> rankList = contentService.getWaitingSort(planAccount);
        if (CollectionUtils.isEmpty(rankList)) {
            return;
        }
        if (StringUtils.isNotEmpty(planAccount.getSortStrategy())) {
            rankList = contentService.getSortList(planAccount);
            if (CollectionUtils.isEmpty(rankList)) {
                return;
            }
        }
        //文章处理  小程序查询和发布
        List<PushContentParam> pushContentList = new ArrayList<>();
        List<Long> sendIds = new ArrayList<>();
        for (PublishArticleData publishArticleData : rankList) {
            PushContentParam contentParam = new PushContentParam();
            PublishContent publishContent = publicContentService.getPublishContent(planAccount, publishArticleData);
            if (publishContent == null) {
                continue;
            }
            //判断是否插入小程序
            if (planAccount.getMiniprogramInsertFlag() != null && planAccount.getMiniprogramInsertFlag() == 1) {
                //获取小程序
                List<PublishMiniprogram> publishMiniprogramList = publicContentService.getPublishMiniprograms(publishContent);
                //不存在则重新生成
                if (CollectionUtils.isEmpty(publishMiniprogramList)) {
                    if (Objects.equals(publishContent.getSourceType(), SourceTypesEnum.longArticleVideoPoolSource.getVal())) {
                        List<VideoDetail> videoDetails = contentService.getOnlyMiniPublishVideoDetail(publishContent);
                        log.info("publishContentId={}, videoDetails={}", publishContent.getId(), videoDetails);
                        if (CollectionUtils.isEmpty(videoDetails)) {
                            continue;
                        }
                        publishMiniprogramList = getPublishMiniprogramList(videoDetails, planAccount, publishContent);
                        if (CollectionUtils.isEmpty(publishMiniprogramList)) {
                            continue;
                        }
                    } else {
                        MatchVideo matchVideo = contentService.getMatchVideo(publishContent, planAccount);
                        if (matchVideo == null) {
                            continue;
                        }
                        List<VideoDetail> videoDetails = contentService.
                                getPublishVideoDetail(matchVideo, PublicFlagEnum.NOT_PUBLIC.getStatusCode(), planAccount, publishContent);
                        log.info("publishContentId={}, videoDetails={}", publishContent.getId(), videoDetails);
                        if (CollectionUtils.isEmpty(videoDetails)) {
                            continue;
                        }
                        publishMiniprogramList = getPublishMiniprogramList(videoDetails, planAccount, publishContent);
                        if (CollectionUtils.isEmpty(publishMiniprogramList)) {
                            continue;
                        }
                        contentService.updateMatchContent(publishContent, publishMiniprogramList, matchVideo);
                    }
                }
                log.info("publishMiniprogramList={}", publishMiniprogramList);
                List<PublishMiniprogramParam> publishCardList = getPublishCardList(publishMiniprogramList);
                contentParam.setMiniprogramCardList(publishCardList);
            }

            sendIds.add(publishContent.getId());
            contentParam.setPublishContentId(publishContent.getPublishContentId());
            pushContentList.add(contentParam);
        }

        if (fwhGhIdList != null && fwhGhIdList.contains(planAccount.getGhId())) {
            pushBatchGroupPushTask(planAccount, pushContentList, sendIds);
        } else {
            CreatePushTaskParam gzhPushParam = getCreatePushTaskParam(planAccount, pushContentList);
            if (gzhPushParam == null) {
                return;
            }
            log.info("gzhPushParam={}", gzhPushParam);
            String pushId = aigcService.createPushTask(gzhPushParam);
            log.info("pushId = {}", pushId);
            if (StringUtils.isNotEmpty(pushId)) {
                publicContentService.updatePublishContentPushId(sendIds, pushId);
            }
        }

    }

    private List<PublishMiniprogram> getPublishMiniprogramList(List<VideoDetail> videoDetails, PlanAccount planAccount,
                                                               PublishContent publishContent) {
        if (CollectionUtils.isEmpty(videoDetails)) {
            return null;
        }
        //获取小程序卡片  判断封面是否可用
        List<PublishMiniprogram> publishMiniprogramList = cardService.generateCards(videoDetails, planAccount, publishContent);
        if (CollectionUtils.isEmpty(publishMiniprogramList)) {
            return null;
        }
//        if (!Objects.equals(publishContent.getSourceType(), SourceTypesEnum.longArticleVideoPoolSource.getVal())) {
//            if (publishMiniprogramList.size() < 2) {
//                return null;
//            }
//        }
        for (PublishMiniprogram publishMiniprogram : publishMiniprogramList) {
            //插入rootSource
            rootSourceService.addRootSource(publishMiniprogram, planAccount, publishContent);
            publishMiniprogramMapper.insertSelective(publishMiniprogram);
        }
        return publishMiniprogramList;
    }

    private CreatePushTaskParam getCreatePushTaskParam(PlanAccount planAccount, List<PushContentParam> pushContentList) {
        CreatePushTaskParam gzhPushParam = new CreatePushTaskParam();
        gzhPushParam.setPlanId(planAccount.getPlanId());
        gzhPushParam.setAccountId(planAccount.getAccountId());
        gzhPushParam.setPushType(planAccount.getPushType());
        gzhPushParam.setPushScheduleTimestamp(System.currentTimeMillis());
        if (planAccount.getIsSpecialSetting() != 1 && StringUtils.isNotEmpty(planAccount.getPublishPushTime())) {
            String dateTimeStr = TimeZoneUtil.todayYMD(TimeZoneUtil.Timezone.china) + " " + planAccount.getPublishPushTime();
            gzhPushParam.setPushScheduleTimestamp(TimeZoneUtil.getDateStrTimestamp(dateTimeStr, "yyyy-MM-dd HH:mm:ss", TimeZoneUtil.Timezone.china));
        } else {
            gzhPushParam.setPushScheduleTimestamp(System.currentTimeMillis());
        }
        //文章列表为空直接返回
        if (CollectionUtils.isEmpty(pushContentList)) {
            return null;
        }
        //自动群发少于最少数量不发送  例外设置数量少于最少发送数量不发送
        if (Objects.equals(planAccount.getPushType(), PublishGzhPushTypeEnum.auto_group_publish.getVal())
                || planAccount.getIsSpecialSetting() == 1) {
            if (pushContentList.size() < planAccount.getPublishPreMinNum()) {
                return null;
            }
        }
        gzhPushParam.setPushContentList(pushContentList);
        return gzhPushParam;
    }

    private void pushBatchGroupPushTask(PlanAccount planAccount, List<PushContentParam> pushContentList, List<Long> sendIds) {
        CreateBatchGroupPushTaskParam gzhPushParam = new CreateBatchGroupPushTaskParam();
        gzhPushParam.setPlanId(planAccount.getPlanId());
        gzhPushParam.setAccountId(planAccount.getAccountId());
        gzhPushParam.setPushType(PublishGzhPushTypeEnum.batch_group_publish.getVal());
        gzhPushParam.setPushScheduleTimestamp(System.currentTimeMillis());
        if (planAccount.getIsSpecialSetting() != 1 && StringUtils.isNotEmpty(planAccount.getPublishPushTime())) {
            String dateTimeStr = TimeZoneUtil.todayYMD(TimeZoneUtil.Timezone.china) + " " + planAccount.getPublishPushTime();
            gzhPushParam.setPushScheduleTimestamp(TimeZoneUtil.getDateStrTimestamp(dateTimeStr, "yyyy-MM-dd HH:mm:ss", TimeZoneUtil.Timezone.china));
        } else {
            gzhPushParam.setPushScheduleTimestamp(System.currentTimeMillis());
        }
        //文章列表为空直接返回
        if (CollectionUtils.isEmpty(pushContentList)) {
            return;
        }
        //自动群发少于最少数量不发送  例外设置数量少于最少发送数量不发送
        if (Objects.equals(planAccount.getPushType(), PublishGzhPushTypeEnum.auto_group_publish.getVal())
                || planAccount.getIsSpecialSetting() == 1) {
            if (pushContentList.size() < planAccount.getPublishPreMinNum()) {
                return;
            }
        }
        int remainingDaysInCurrentMonth = DateUtil.getRemainingDaysInCurrentMonth();
        Integer remainingCount;
        List<String> sendOpenIds = new ArrayList<>();
        if (remainingDaysInCurrentMonth <= 4) {
            remainingCount = articleUseGroupMapper.selectRemainingCountByGzhId(planAccount.getGhId(), remainingDaysInCurrentMonth);
            List<String> remainingCountOpenIds = articleUseGroupMapper.selectOpenIdsByRemainingCount(planAccount.getGhId(), remainingDaysInCurrentMonth);
            sendOpenIds.addAll(remainingCountOpenIds);
        } else {
            remainingCount = articleUseGroupMapper.selectRemainingCountByGzhId(planAccount.getGhId(), null);
        }
        if (remainingDaysInCurrentMonth > 0 && remainingCount != null && remainingCount / remainingDaysInCurrentMonth > 0) {
            Integer count = articleUseGroupMapper.selectFansBeforePublishCount(planAccount.getGhId());
            FwhDailyPublishDetail fwhDailyPublishDetail = new FwhDailyPublishDetail();
            fwhDailyPublishDetail.setAccountName(planAccount.getAccountName());
            fwhDailyPublishDetail.setGhId(planAccount.getGhId());
            fwhDailyPublishDetail.setPublishDate(DateUtil.getThatDayDateString());
            fwhDailyPublishDetail.setFansBeforePublish(count);
            fwhDailyPublishDetail.setPublishTimesBeforePublish(remainingCount);
            fwhDailyPublishDetailMapper.insertSelective(fwhDailyPublishDetail);
            List<String> needSendOpenIds = articleUseGroupMapper.selectOpenIds(planAccount.getGhId(), remainingCount / remainingDaysInCurrentMonth);
            sendOpenIds.addAll(needSendOpenIds);
        }
        if (CollectionUtils.isEmpty(sendOpenIds)) {
            return;
        }
        int batchSize = 10000; // 每组大小
        int totalSize = sendOpenIds.size();
        List<PublishContent> publishContentList = publicContentService.getPublishContentById(sendIds);
        int group = 1;
        for (int i = 0; i < totalSize; i += batchSize) {
            int toIndex = Math.min(i + batchSize, totalSize);
            // 获取当前批次的数据
            int batchInsertSize = 1000;
            List<GroupSendOpenId> batchInsertList = new ArrayList<>(batchInsertSize);
            List<String> batch = sendOpenIds.subList(i, toIndex);
            for (String openId : batch) {
                GroupSendOpenId groupSendOpenId = new GroupSendOpenId();
                groupSendOpenId.setGhId(planAccount.getGhId());
                groupSendOpenId.setPublishDate(DateUtil.getThatDayDateString());
                groupSendOpenId.setUserGroupId(group);
                groupSendOpenId.setOpenId(openId);
                batchInsertList.add(groupSendOpenId);
                // 当批次大小达到1000时执行插入，并清空列表
                if (batchInsertList.size() >= batchInsertSize) {
                    groupSendOpenIdMapper.batchInsert(batchInsertList);
                    batchInsertList.clear();
                }
            }
            // 处理剩余的记录（如果有）
            if (!batchInsertList.isEmpty()) {
                groupSendOpenIdMapper.batchInsert(batchInsertList);
            }


            List<CreateBatchGroupItemParam> createBatchGroupItemParams = new ArrayList<>();
            CreateBatchGroupItemParam createBatchGroupItemParam = new CreateBatchGroupItemParam();
            createBatchGroupItemParam.setPushContentList(pushContentList);
            createBatchGroupItemParam.setUserGroupId(String.valueOf(group));
            createBatchGroupItemParam.setOpenIds(batch);
            createBatchGroupItemParams.add(createBatchGroupItemParam);
            gzhPushParam.setBatchGroupItems(createBatchGroupItemParams);
            log.info("gzhPushParam={}", gzhPushParam);
            String pushId = aigcService.createBatchGroupPushTask(gzhPushParam);
            log.info("pushId = {}", pushId);
            for (PublishContent publishContent : publishContentList) {
                PublishContent publishContent1 = new PublishContent();
                BeanUtils.copyProperties(publishContent, publishContent1);
                publishContent1.setId(null);
                publishContent1.setPushId(pushId);
                publishContent1.setStatus(1);
                publishContent1.setUserGroupId(group);
                publishContent1.setRootId(publishContent.getId());
                publishContent1.setCreateTime(null);
                publishContent1.setUpdateTime(null);
                publishContentMapper.insertSelective(publishContent1);
                GroupSendResult groupSendResult = new GroupSendResult();
                groupSendResult.setAccountName(planAccount.getAccountName());
                groupSendResult.setGhId(planAccount.getGhId());
                groupSendResult.setUserGroupId(group);
                groupSendResult.setContentId(publishContent.getSourceId());
                groupSendResult.setPushId(pushId);
                groupSendResult.setTraceId(publishContent.getTraceId());
                groupSendResult.setPublishContentId(publishContent.getPublishContentId());
                groupSendResult.setPublishDate(DateUtil.getThatDayDateString());
                groupSendResult.setSentCount(batch.size());
                groupSendResultMapper.insertSelective(groupSendResult);
            }
            group++;
        }
    }

    private List<PublishMiniprogramParam> getPublishCardList(List<PublishMiniprogram> publishMiniprogramList) {
        List<PublishMiniprogramParam> publishCardList = new ArrayList<>();
        for (PublishMiniprogram publishMiniprogram : publishMiniprogramList) {
            PublishMiniprogramParam param = new PublishMiniprogramParam();
            param.setProgramId(publishMiniprogram.getAppId());
            param.setProgramName(publishMiniprogram.getAppName());
            param.setProgramAvatar(publishMiniprogram.getAvatar());
            param.setMini_id(publishMiniprogram.getMiniId());
            param.setMiniprogramUseType(publishMiniprogram.getMiniProgramType());
            String videoTitle = publishMiniprogram.getVideoTitle();
            if (org.springframework.util.StringUtils.hasText(videoTitle)) {
                videoTitle = videoTitle.replace("\"", "");
                if (videoTitle.length() > 32) {
                    videoTitle = videoTitle.substring(0, 32);
                }
                param.setProductionName(videoTitle);
            }
            param.setProductionCover(publishMiniprogram.getWxUrl());
            param.setProductionPath(publishMiniprogram.getProductionPath());
            param.setRootShareId(publishMiniprogram.getRootShareId());
            param.setSource(publishMiniprogram.getSource());
            param.setVideoUrl(publishMiniprogram.getVideoPath());
            param.setParagraphPosition(BigDecimal.valueOf(publishMiniprogram.getIndex() * 0.25));
            if (publishMiniprogram.getIndex() == 1) {
                param.setParagraphPosition(BigDecimal.valueOf(0.01));
            }
            param.setTraceId(publishMiniprogram.getTraceId());
            publishCardList.add(param);
        }
        return publishCardList;
    }

    @Override
    public void getPushStatus() {
        PublishContentExample example = new PublishContentExample();
        example.createCriteria().andStatusEqualTo(1).andPushIdIsNotNull().andCreateTimeGreaterThan(DateUtil.getThatDayDate());
        List<PublishContent> publishContents = publishContentMapper.selectByExample(example);
        Map<String, List<PublishContent>> groupedByPushId = publishContents.stream()
                .filter(content -> content.getPushId() != null) // 过滤掉 pushId 为空的情况
                .collect(Collectors.groupingBy(PublishContent::getPushId));

        for (Map.Entry<String, List<PublishContent>> entry : groupedByPushId.entrySet()) {
            String pushId = entry.getKey();
            PushStatusParam param = new PushStatusParam();
            param.setPushId(pushId);
            PushStatusVO pushStatusVO = aigcService.queryPushStatus(param);
            log.info("pushStatusVO={} param={}", pushStatusVO, param);
            if (pushStatusVO == null) {
                continue;
            }
            int updateStatus = 0;
            //推送失败
            if (pushStatusVO.getPushStatus() == 3) {
                updateStatus = 3;
            }
            //推送成功
            if (pushStatusVO.getPushStatus() == 2) {
                //发表失败
                if (pushStatusVO.getPublishStatus() == 3) {
                    updateStatus = 3;
//                    if (StringUtils.isNotEmpty(pushStatusVO.getErrorCode())) {
//                        int errorCode = Integer.parseInt(pushStatusVO.getErrorCode());
//                        if (errorCode == 6 || errorCode == 7 || errorCode == 8 || errorCode == 999) {
//                            updateStatus = 2;
//                        }
//                    }

                }
                //发表成功
                if (pushStatusVO.getPublishStatus() == 2) {
                    updateStatus = 2;
                }
            }


            Long planAccountId = entry.getValue().get(0).getPlanAccountId();
            if (planAccountId == null) {
                continue;
            }
            PlanAccount planAccount = planAccountService.getPlanAccountById(planAccountId);
            if (updateStatus == 2) {
                for (PublishContent publishContent : entry.getValue()) {
                    publicContentService.updatePublishContentStatus(updateStatus, publishContent.getId(), pushStatusVO.getErrorMsg(), pushId);
                    if (Objects.equals(publishContent.getSourceType(), SourceTypesEnum.longArticleVideoPoolSource.getVal())) {
                        List<PublishMiniprogram> publishMiniprogramList = publicContentService.getPublishMiniprograms(publishContent);
                        if (CollectionUtils.isEmpty(publishMiniprogramList)) {
                            continue;
                        }
                        for (PublishMiniprogram publishMiniprogram : publishMiniprogramList) {
                            videoService.addOffVideo(publishMiniprogram);
                        }
                    }
                }
                if (planAccount == null) {
                    continue;
                }
                if (Objects.equals(planAccount.getPushType(), PublishGzhPushTypeEnum.auto_group_publish.getVal()) ||
                        Objects.equals(planAccount.getPushType(), PublishGzhPushTypeEnum.batch_group_publish.getVal())) {
                    planAccountService.updateStatus(2, planAccountId);
                }
                if (fwhGhIdList != null && fwhGhIdList.contains(planAccount.getGhId())) {
                    GroupSendResultExample groupSendResultExample = new GroupSendResultExample();
                    groupSendResultExample.createCriteria().andPushIdEqualTo(pushId);
                    List<GroupSendResult> groupSendResults = groupSendResultMapper.selectByExample(groupSendResultExample);
                    if (!CollectionUtils.isEmpty(groupSendResults)) {
                        GroupSendResult updateGroupSendResult = new GroupSendResult();
                        updateGroupSendResult.setStatus(updateStatus);
                        groupSendResultMapper.updateByExampleSelective(updateGroupSendResult, groupSendResultExample);
                        Set<String> groupKeys = groupSendResults.stream()
                                .map(e -> e.getGhId() + "&&" + e.getPublishDate() + "&&" + e.getUserGroupId())
                                .collect(Collectors.toSet());
                        for (String groupKey : groupKeys) {
                            String[] groupArray = groupKey.split("&&");
                            String ghId = groupArray[0];
                            String publishDate = groupArray[1];
                            String userGroupId = groupArray[2];
                            GroupSendOpenIdExample groupSendOpenIdExample = new GroupSendOpenIdExample();
                            groupSendOpenIdExample.createCriteria().andGhIdEqualTo(ghId)
                                    .andPublishDateEqualTo(publishDate)
                                    .andUserGroupIdEqualTo(Integer.parseInt(userGroupId));
                            List<GroupSendOpenId> groupSendOpenIds = groupSendOpenIdMapper.selectByExample(groupSendOpenIdExample);
                            if (!CollectionUtils.isEmpty(groupSendOpenIds)) {
                                for (GroupSendOpenId groupSendOpenId : groupSendOpenIds) {
                                    ArticleUseGroupExample articleUseGroupExample = new ArticleUseGroupExample();
                                    articleUseGroupExample.createCriteria().andGzhIdEqualTo(groupSendOpenId.getGhId())
                                            .andOpenIdEqualTo(groupSendOpenId.getOpenId());
                                    List<ArticleUseGroup> articleUseGroups = articleUseGroupMapper.selectByExample(articleUseGroupExample);
                                    if (!CollectionUtils.isEmpty(articleUseGroups)) {
                                        for (ArticleUseGroup articleUseGroup : articleUseGroups) {
                                            ArticleUseGroup updateArticleUseGroup = new ArticleUseGroup();
                                            updateArticleUseGroup.setId(articleUseGroup.getId());
                                            updateArticleUseGroup.setRemainingCount(articleUseGroup.getRemainingCount() - 1);
                                            articleUseGroupMapper.updateByPrimaryKeySelective(updateArticleUseGroup);
                                        }
                                    }
                                }
                            }
                        }

                    }
                }
            }
            if (updateStatus == 3) {
                PublishContentExample publishContentExample = new PublishContentExample();
                publishContentExample.createCriteria().andPushIdEqualTo(pushId);
                PublishContent update = new PublishContent();
                if (StringUtils.isNotEmpty(pushStatusVO.getErrorMsg())) {
                    update.setReason(pushStatusVO.getErrorMsg());
                }
                update.setStatus(updateStatus);
                publishContentMapper.updateByExampleSelective(update, publishContentExample);
                if (planAccount == null) {
                    continue;
                }
                planAccountService.updateRetry(planAccount);
                if (Objects.equals(planAccount.getPushType(), PublishGzhPushTypeEnum.auto_group_publish.getVal())) {
                    if (StringUtils.isNotEmpty(pushStatusVO.getErrorMsg())) {
                        //45028群发配额用完不再重试  48004封号账号也不再重试
                        if (pushStatusVO.getErrorMsg().contains("45028") || pushStatusVO.getErrorMsg().contains("48004")) {
                            planAccountService.updateStatus(3, planAccountId);
                        }
                    }
                }
            }
            if (StringUtils.isNotEmpty(pushStatusVO.getErrorMsg()) && !pushStatusVO.getErrorMsg().contains("45028")) {
                log.error("push error pushId={} msg={}", pushId, pushStatusVO.getErrorMsg());
                String message = String.format("发布失败，请查看，pushId=%s 失败信息：%s 账号名称：%s 计划名称：%s",
                        pushId, pushStatusVO.getErrorMsg(), planAccount.getAccountName(), planAccount.getPlanName());
                LarkRobotUtil.sendMessage(message);
            }
        }
    }
}
