package com.tzld.piaoquan.longarticle.service.local.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.odps.data.Record;
import com.tzld.piaoquan.longarticle.common.enums.ContentResultStatusEnum;
import com.tzld.piaoquan.longarticle.common.enums.ContentStatusEnum;
import com.tzld.piaoquan.longarticle.common.enums.PublicFlagEnum;
import com.tzld.piaoquan.longarticle.component.RedisLock;
import com.tzld.piaoquan.longarticle.dao.mapper.aigc.ProducePlanExeRecordMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.*;
import com.tzld.piaoquan.longarticle.model.bo.MatchContent;
import com.tzld.piaoquan.longarticle.model.bo.VideoDetail;
import com.tzld.piaoquan.longarticle.model.dto.AuditContentRequest;
import com.tzld.piaoquan.longarticle.model.po.aigc.ProducePlanExeRecord;
import com.tzld.piaoquan.longarticle.model.po.aigc.ProducePlanExeRecordExample;
import com.tzld.piaoquan.longarticle.model.po.longarticle.*;
import com.tzld.piaoquan.longarticle.service.local.KimiService;
import com.tzld.piaoquan.longarticle.service.remote.MatchService;
import com.tzld.piaoquan.longarticle.utils.DateUtil;
import com.tzld.piaoquan.longarticle.utils.LarkRobotUtil;
import com.tzld.piaoquan.longarticle.utils.OdpsUtil;
import com.tzld.piaoquan.longarticle.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class MatchVideoServiceImpl {


    private static final String KIMI_LOCK_KEY = "kimi_lock_key_%s";

    private static final String CRAWLER_LOCK_KEY = "crawler_lock_key_%s";

    private static final String UPLOAD_CRAWLER_LOCK_KEY = "upload_crawler_lock_key_%s";

    private static final String CRAWLER_FAIL_COUNT_KEY = "crawler_count_key_%s";

    private static final String UPLOAD_CRAWLER_FAIL_COUNT_KEY = "upload_crawler_count_key_%s";

    private static final String KIMI_FAIL_COUNT_KEY = "kimi_count_key_%s";

    private static final String EXIST_RESULT_KEY = "exist_result_key_%s";

    private static final String SINGLE_VIDEO_LOCK_KEY = "single_video_lock_key_%s";


    @Autowired
    KimiService kimiService;

    @Autowired
    private MatchVideoMapper matchVideoMapper;

    @Autowired
    private LongArticlesTextMapper longArticlesTextMapper;

    @Autowired
    private CrawlerVideoServiceImpl crawlerVideoService;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private ProducePlanExeRecordMapper producePlanExeRecordMapper;

    @Autowired
    private PromotionSourceMapper promotionSourceMapper;

    @Autowired
    private ContentServiceImpl contentService;

    @Autowired
    private SingleVideoSourceMapper singleVideoSourceMapper;

    @Autowired
    private VideoContentUnderstandingMapper videoContentUnderstandingMapper;

    @Autowired
    private CrawlerVideoMapper crawlerVideoMapper;

    @Autowired
    private MatchService matchService;

    @Autowired
    private PiaoquanSourceVideoPoolMapper piaoquanSourceVideoPoolMapper;


    // 定义一个阻塞队列
    private static final ArrayBlockingQueue<MatchVideo> matchKimiVideoQueue = new ArrayBlockingQueue<>(1000000);
    // 定义一个线程池，设置消费线程的数量
    private static final ThreadPoolExecutor matchKimiVideoPoolExecutor = (ThreadPoolExecutor) Executors.newFixedThreadPool(5);


    // 定义一个阻塞队列
    private static final ArrayBlockingQueue<MatchVideo> matchCrawlerVideoQueue = new ArrayBlockingQueue<>(10000000);
    // 定义一个线程池，设置消费线程的数量
    private static final ThreadPoolExecutor matchCrawlerVideoPoolExecutor = (ThreadPoolExecutor) Executors.newFixedThreadPool(10);

    // 定义一个阻塞队列
    private static final ArrayBlockingQueue<MatchVideo> uploadCrawlerVideoQueue = new ArrayBlockingQueue<>(1000000);
    // 定义一个线程池，设置消费线程的数量
    private static final ThreadPoolExecutor uploadCrawlerVideoPoolExecutor = (ThreadPoolExecutor) Executors.newFixedThreadPool(10);

    @Transactional
    public void addMatchVideo(MatchContent matchContent) {
        MatchVideoExample example = new MatchVideoExample();
        example.createCriteria().andContentIdEqualTo(matchContent.getSourceId())
                .andGhIdEqualTo(matchContent.getGhId())
                .andPublishFlagEqualTo(matchContent.getPublishFlag());
        long l = matchVideoMapper.countByExample(example);
        if (l > 0) {
            return;
        }
        String traceId = "search-" + UUID.randomUUID();
        MatchVideo matchVideo = new MatchVideo();
        matchVideo.setTraceId(traceId);
        matchVideo.setContentId(matchContent.getSourceId());
        matchVideo.setAccountName(matchContent.getAccountName());
        matchVideo.setGhId(matchContent.getGhId());
        matchVideo.setFlowPoolLevel(matchContent.getFlowPoolLevelTag());
        matchVideo.setContentStatus(0);
        matchVideo.setPublishFlag(matchContent.getPublishFlag());
        long timestamp = System.currentTimeMillis() / 1000;
        matchVideo.setContentStatusUpdateTime(Long.valueOf(timestamp).intValue());
        matchVideo.setRequestTimestamp(Long.valueOf(timestamp).intValue());
        matchVideoMapper.insertSelective(matchVideo);
        LongArticlesTextExample longArticlesTextExample = new LongArticlesTextExample();
        longArticlesTextExample.createCriteria().andContentIdEqualTo(matchContent.getSourceId());
        long l1 = longArticlesTextMapper.countByExample(longArticlesTextExample);
        if (l1 == 0) {
            LongArticlesText longArticlesText = new LongArticlesText();
            longArticlesText.setArticleTitle(matchContent.getTitle());
            longArticlesText.setArticleText(matchContent.getContent());
            longArticlesText.setContentId(matchContent.getSourceId());
            longArticlesText.setKimiStatus(0);
            longArticlesTextMapper.insertSelective(longArticlesText);
        }
    }

    public void kimiContent() throws InterruptedException {
        if (matchKimiVideoPoolExecutor.getCorePoolSize() - matchKimiVideoPoolExecutor.getActiveCount() > 0) {
            int threadSize = matchKimiVideoPoolExecutor.getCorePoolSize() - matchKimiVideoPoolExecutor.getActiveCount();
            log.info("threadNum={}", threadSize);
            CountDownLatch countDownLatch = new CountDownLatch(threadSize);
            // 启动消费者线程
            for (int i = 0; i < threadSize; i++) {
                matchKimiVideoPoolExecutor.execute(new Thread(() -> {
                    log.info("启动Kimi搜索线程");
                    while (true) {
                        try {
                            // 超过 2 分钟没有数据，销毁当前线程
                            MatchVideo matchVideo = matchKimiVideoQueue.poll(2, TimeUnit.MINUTES); // 等待最多 2 分钟
                            log.info("matchKimiVideoQueue size={}", matchKimiVideoQueue.size());
                            if (matchVideo == null) {
                                break; // 退出当前线程
                            }
                            processKimiMatchContent(matchVideo);
                        } catch (Exception e) {
                            log.error("Kimi搜索线程异常", e);
                        }
                    }
                    log.info("启动Kimi搜索线程结束");
                    countDownLatch.countDown();
                }));
            }
            List<Integer> targetStatus = Arrays.asList(0, 1, 2);
            List<MatchVideo> matchVideos;
            //循环增加游标位置，通过主键索引过滤已经处理的数据
            do {
                Integer id = (Integer) redisTemplate.opsForValue().get("last_match_video_id");
                if (id == null) {
                    redisTemplate.opsForValue().set("last_match_video_id", 12864912);
                    id = 12864912;
                }
                MatchVideoExample example = new MatchVideoExample();
                example.createCriteria().andIdGreaterThan(id);
                example.setOrderByClause("id asc");
                Page<Object> page = new Page<>();
                page.setCurrentPage(1);
                page.setPageSize(1000);
                example.setPage(page);
                matchVideos = matchVideoMapper.selectByExample(example);
                if (CollectionUtils.isEmpty(matchVideos)) {
                    break;
                }
                boolean flag = true;
                for (MatchVideo matchVideo : matchVideos) {
                    if (targetStatus.contains(matchVideo.getContentStatus())) {
                        flag = false;
                        break;
                    }
                    //需要发布的状态需要到4才能结束
                    if (matchVideo.getContentStatus() == ContentStatusEnum.SUCCESS_3.getStatusCode() && matchVideo.getPublishFlag() == 1) {
                        flag = false;
                        break;
                    }
                }
                if (flag) {
                    Integer lastId = matchVideos.get(matchVideos.size() - 1).getId();
                    redisTemplate.opsForValue().set("last_match_video_id", lastId);
                } else {
                    break;
                }
            } while (!CollectionUtils.isEmpty(matchVideos));

            Integer id = (Integer) redisTemplate.opsForValue().get("last_match_video_id");
            if (id != null) {
                do {
                    MatchVideoExample example = new MatchVideoExample();
                    example.createCriteria().andIdGreaterThan(id).andContentStatusEqualTo(0);
                    example.setOrderByClause("id asc");
                    Page<Object> page = new Page<>();
                    page.setCurrentPage(1);
                    page.setPageSize(1000);
                    example.setPage(page);
                    matchVideos = matchVideoMapper.selectByExample(example);
                    if (CollectionUtils.isEmpty(matchVideos)) {
                        break;
                    }
                    id = matchVideos.get(matchVideos.size() - 1).getId();
                    matchKimiVideoQueue.addAll(matchVideos);
                } while (!CollectionUtils.isEmpty(matchVideos));
            }
            countDownLatch.await();
        }
    }

    public void addCrawlerVideo(MatchVideo matchVideo) {

        //判断数据是否已经存在
        CrawlerVideoExample crawlerVideoExample = new CrawlerVideoExample();
        crawlerVideoExample.createCriteria().andContentIdEqualTo(matchVideo.getContentId());
        long l = crawlerVideoMapper.countByExample(crawlerVideoExample);
        if (l > 0) {
            return;
        }

        //数据准备
        SingleVideoSourceExample example = new SingleVideoSourceExample();
        example.createCriteria().andContentTraceIdEqualTo(matchVideo.getVideoPoolTraceId());
        List<SingleVideoSource> singleVideoSources = singleVideoSourceMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(singleVideoSources)) {
            return;
        }
        SingleVideoSource singleVideoSource = singleVideoSources.get(0);

        //视频抓取表插入
        CrawlerVideo crawlerVideo = new CrawlerVideo();
        crawlerVideo.setDownloadStatus(2);
        crawlerVideo.setVideoTitle(singleVideoSource.getArticleTitle());
        crawlerVideo.setCoverUrl(singleVideoSource.getCoverUrl());
        crawlerVideo.setVideoOssPath(singleVideoSource.getVideoOssPath());
        crawlerVideo.setCrawlerTime(new Date(singleVideoSource.getCrawlerTimestamp() * 1000));
        crawlerVideo.setContentId(matchVideo.getContentId());
        crawlerVideo.setPlatform(singleVideoSource.getPlatform());
        crawlerVideoMapper.insertSelective(crawlerVideo);

        //kimi插入
        LongArticlesTextExample longArticlesTextExample = new LongArticlesTextExample();
        longArticlesTextExample.createCriteria().andContentIdEqualTo(matchVideo.getContentId());
        List<LongArticlesText> longArticlesTexts = longArticlesTextMapper.selectByExampleWithBLOBs(longArticlesTextExample);
        LongArticlesText longArticlesText = longArticlesTexts.get(0);
        longArticlesText.setStartProcessingTime(new Date());
        longArticlesText.setKimiTitle(singleVideoSource.getMiniProgramTitle());
        longArticlesText.setKimiStatus(ContentResultStatusEnum.SUCCESS.getStatus());
        longArticlesText.setResultStatus(ContentResultStatusEnum.SUCCESS.getStatus());
        longArticlesTextMapper.updateByPrimaryKeyWithBLOBs(longArticlesText);

        //修改视频匹配表状态
        matchVideo.setContentStatus(ContentStatusEnum.SUCCESS_3.getStatusCode());
        matchVideoMapper.updateByPrimaryKey(matchVideo);


        //增加审核
        AuditContentRequest auditContentRequest = new AuditContentRequest();
        auditContentRequest.setContentId(matchVideo.getContentId());
        auditContentRequest.setFlowPoolLevel(matchVideo.getFlowPoolLevel());
        boolean addAudit = matchService.addAudit(auditContentRequest);
        if (!addAudit) {
            LarkRobotUtil.sendMessage("添加审核文章失败 contentId=" + matchVideo.getContentId() +
                    " flowPoolLevel=" + matchVideo.getFlowPoolLevel());
        }


    }

    public void processKimiMatchContent(MatchVideo matchVideo) {
        if (matchVideo.getContentStatus() != ContentStatusEnum.DEFAULT.getStatusCode()) {
            return;
        }

        if (StringUtils.isNotEmpty(matchVideo.getVideoPoolTraceId())) {
            //已经存在结果  直接设置
            if (existMatchVideoResult(matchVideo)) {
                return;
            }
            String lockKey = String.format(SINGLE_VIDEO_LOCK_KEY, matchVideo.getContentId());
            String lockValue = UUID.randomUUID().toString();
            boolean lock = redisLock.tryLock(lockKey, lockValue, 300, TimeUnit.SECONDS);
            if (lock) {
                log.info("singleCrawlerVideo lock lockKey={}", lockKey);
                addCrawlerVideo(matchVideo);
                log.info("singleCrawlerVideo unlock lockKey={}", lockKey);
                redisLock.unlock(lockKey, lockValue);
            }
            return;
        }

        //1.执行kimi任务
        LongArticlesText kimiText = kimiService.getKimiText(matchVideo.getContentId());
        if (kimiText == null) {
            return;
        }
        //执行kimi任务
        if (kimiText.getKimiStatus() == 0) {
            //已经存在结果  直接设置
            if (existMatchVideoResult(matchVideo)) {
                return;
            }
            String lockKey = String.format(KIMI_LOCK_KEY, matchVideo.getContentId());
            String lockValue = UUID.randomUUID().toString();
            boolean lock = redisLock.tryLock(lockKey, lockValue, 300, TimeUnit.SECONDS);
            if (lock) {
                log.info("processKimiMatchContent lock lockKey={}", lockKey);
                addStartProcessingTime(matchVideo.getContentId());
                boolean res = kimiService.updateKimiContent(matchVideo.getContentId());
                if (res) {
                    updateStatus(matchVideo.getId(), ContentStatusEnum.KIMI_SUCCESS.getStatusCode());
                } else {
                    String countKey = String.format(KIMI_FAIL_COUNT_KEY, matchVideo.getContentId());
                    Integer count = (Integer) redisTemplate.opsForValue().get(countKey);
                    if (count != null && count >= 3) {
                        //更新状态为失败
                        LarkRobotUtil.sendMessage("kimiService updateKimiContent fail contentId=" + matchVideo.getContentId());
                        log.error("kimiService updateKimiContent fail contentId={}", matchVideo.getContentId());
                        kimiService.updateKimiContentStatus(matchVideo.getContentId(), 2);
                        failMatchVideo(matchVideo);
                    } else {
                        if (count == null) {
                            redisTemplate.opsForValue().set(countKey, 1, 3, TimeUnit.DAYS);
                        } else {
                            redisTemplate.opsForValue().set(countKey, count + 1, 3, TimeUnit.DAYS);
                        }
                    }
                }
                log.info("processKimiMatchContent unlock lockKey={}", lockKey);
                redisLock.unlock(lockKey, lockValue);
            }
        } else if (kimiText.getKimiStatus() == 1) {
            //更新状态为kimi执行完成
            updateStatus(matchVideo.getId(), ContentStatusEnum.KIMI_SUCCESS.getStatusCode());
        } else if (kimiText.getKimiStatus() == 2) {
            failMatchVideo(matchVideo);
        }

    }

    public void matchCrawlerVideo() throws InterruptedException {
        if (matchCrawlerVideoPoolExecutor.getCorePoolSize() - matchCrawlerVideoPoolExecutor.getActiveCount() > 0) {
            int threadSize = matchCrawlerVideoPoolExecutor.getCorePoolSize() - matchCrawlerVideoPoolExecutor.getActiveCount();
            log.info("threadNum={}", threadSize);
            CountDownLatch countDownLatch = new CountDownLatch(threadSize);
            // 启动消费者线程
            for (int i = 0; i < threadSize; i++) {
                matchCrawlerVideoPoolExecutor.execute(new Thread(() -> {
                    log.info("启动匹配小程序线程");
                    while (true) {
                        try {
                            // 超过 2 分钟没有数据，销毁当前线程
                            MatchVideo matchVideo = matchCrawlerVideoQueue.poll(2, TimeUnit.MINUTES); // 等待最多 2 分钟
                            log.info("matchCrawlerVideoQueue size={}", matchCrawlerVideoQueue.size());
                            if (matchVideo == null) {
                                break; // 退出当前线程
                            }
                            processCrawlerMatchContent(matchVideo);
                        } catch (Exception e) {
                            log.error("匹配小程序线程异常", e);
                        }
                    }
                    log.info("启动匹配小程序线程结束");
                    countDownLatch.countDown();
                }));
            }
            List<MatchVideo> matchVideos;
            Integer id = (Integer) redisTemplate.opsForValue().get("last_match_video_id");
            if (id != null) {
                do {
                    MatchVideoExample example = new MatchVideoExample();
                    example.createCriteria().andIdGreaterThan(id).andContentStatusEqualTo(1);
                    example.setOrderByClause("id asc");
                    Page<Object> page = new Page<>();
                    page.setCurrentPage(1);
                    page.setPageSize(1000);
                    example.setPage(page);
                    matchVideos = matchVideoMapper.selectByExample(example);
                    if (CollectionUtils.isEmpty(matchVideos)) {
                        break;
                    }
                    id = matchVideos.get(matchVideos.size() - 1).getId();
                    matchCrawlerVideoQueue.addAll(matchVideos);
                } while (!CollectionUtils.isEmpty(matchVideos));
            }
            countDownLatch.await();
        }
    }


    public void processCrawlerMatchContent(MatchVideo matchVideo) {
        if (matchVideo.getContentStatus() != ContentStatusEnum.KIMI_SUCCESS.getStatusCode()) {
            return;
        }
        boolean existCrawlerVideo = crawlerVideoService.existCrawlerVideo(matchVideo.getContentId());
        log.info("processCrawlerMatchContent contentId={} existCrawlerVideo={}", matchVideo.getContentId(), existCrawlerVideo);
        if (!existCrawlerVideo) {
            //已经存在结果  直接设置
            if (existMatchVideoResult(matchVideo)) {
                return;
            }
            String lockKey = String.format(CRAWLER_LOCK_KEY, matchVideo.getContentId());
            String lockValue = UUID.randomUUID().toString();
            boolean lock = redisLock.tryLock(lockKey, lockValue, 20, TimeUnit.MINUTES);
            if (lock) {
                LongArticlesText kimiText = kimiService.getKimiText(matchVideo.getContentId());
                if (kimiText.getKimiStatus() == 1) {
                    String rootContentId = getRootContentId(matchVideo.getContentId());
                    boolean res = crawlerVideoService.addCrawlerVideo(matchVideo.getContentId(), rootContentId, kimiText);
                    if (res) {
                        updateStatus(matchVideo.getId(), ContentStatusEnum.CRAWLER_SUCCESS.getStatusCode());
                    } else {
                        //匹配失败记录
                        String countKey = String.format(CRAWLER_FAIL_COUNT_KEY, matchVideo.getContentId());
                        Integer count = (Integer) redisTemplate.opsForValue().get(countKey);
                        if (count != null && count >= 3) {
                            //更新状态为失败
//                            LarkRobotUtil.sendMessage("crawlerVideoService addCrawlerVideo fail contentId=" + matchVideo.getContentId());
                            log.error("crawlerVideoService addCrawlerVideo fail contentId={}", matchVideo.getContentId());
                            failMatchVideo(matchVideo);
                        } else {
                            if (count == null) {
                                redisTemplate.opsForValue().set(countKey, 1, 3, TimeUnit.DAYS);
                            } else {
                                redisTemplate.opsForValue().set(countKey, count + 1, 3, TimeUnit.DAYS);
                            }
                        }
                    }
                } else if (kimiText.getKimiStatus() == 2) {
                    failMatchVideo(matchVideo);
                }
                redisLock.unlock(lockKey, lockValue);
            }
        } else {
            //更新状态为爬虫执行完成
            updateStatus(matchVideo.getId(), ContentStatusEnum.CRAWLER_SUCCESS.getStatusCode());
        }
    }

    private String getRootContentId(String contentId) {
        ProducePlanExeRecordExample example = new ProducePlanExeRecordExample();
        example.createCriteria().andPlanExeIdEqualTo(contentId);
        List<ProducePlanExeRecord> producePlanExeRecords = producePlanExeRecordMapper.selectByExample(example);
        if (!CollectionUtils.isEmpty(producePlanExeRecords)) {
            String channelContentId = producePlanExeRecords.get(0).getChannelContentId();
            if (channelContentId == null) {
                return null;
            }
            PromotionSourceExample promotionSourceExample = new PromotionSourceExample();
            promotionSourceExample.createCriteria().andChannelContentIdEqualTo(channelContentId);
            List<PromotionSource> promotionSources = promotionSourceMapper.selectByExample(promotionSourceExample);
            if (CollectionUtils.isEmpty(promotionSources)) {
                return null;
            }
            return promotionSources.get(0).getRootProduceContentId();
        }
        return null;
    }

    public void uploadCrawlerVideo() throws InterruptedException {
        if (uploadCrawlerVideoPoolExecutor.getCorePoolSize() - uploadCrawlerVideoPoolExecutor.getActiveCount() > 0) {
            int threadSize = uploadCrawlerVideoPoolExecutor.getCorePoolSize() - uploadCrawlerVideoPoolExecutor.getActiveCount();
            log.info("threadNum={}", threadSize);
            CountDownLatch countDownLatch = new CountDownLatch(threadSize);
            // 启动消费者线程
            for (int i = 0; i < threadSize; i++) {
                uploadCrawlerVideoPoolExecutor.execute(new Thread(() -> {
                    log.info("启动上传小程序视频线程");
                    while (true) {
                        try {
                            // 超过 2 分钟没有数据，销毁当前线程
                            MatchVideo matchVideo = uploadCrawlerVideoQueue.poll(2, TimeUnit.MINUTES); // 等待最多 2 分钟
                            log.info("uploadCrawlerVideoQueue size={}", uploadCrawlerVideoQueue.size());
                            if (matchVideo == null) {
                                break; // 退出当前线程
                            }
                            processUploadCrawlerVideo(matchVideo);
                        } catch (Exception e) {
                            log.error("上传小程序视频线程异常", e);
                        }
                    }
                    log.info("上传小程序视频线程结束");
                    countDownLatch.countDown();
                }));
            }


            List<MatchVideo> matchVideos;
            Integer id = (Integer) redisTemplate.opsForValue().get("last_match_video_id");
            if (id != null) {
                do {
                    MatchVideoExample example = new MatchVideoExample();
                    example.createCriteria().andIdGreaterThan(id)
                            .andContentStatusEqualTo(ContentStatusEnum.CRAWLER_SUCCESS.getStatusCode());
                    example.setOrderByClause("id asc");
                    Page<Object> page = new Page<>();
                    page.setCurrentPage(1);
                    page.setPageSize(1000);
                    example.setPage(page);
                    matchVideos = matchVideoMapper.selectByExample(example);
                    if (CollectionUtils.isEmpty(matchVideos)) {
                        break;
                    }
                    id = matchVideos.get(matchVideos.size() - 1).getId();
                    uploadCrawlerVideoQueue.addAll(matchVideos);
                } while (!CollectionUtils.isEmpty(matchVideos));
            }
            log.info("uploadCrawlerVideoQueue add end");
            countDownLatch.await();
        }

    }

    public void processUploadCrawlerVideo(MatchVideo matchVideo) {
        log.info("processUploadCrawlerVideo matchVideo={}", matchVideo);
        if (matchVideo.getContentStatus() != ContentStatusEnum.CRAWLER_SUCCESS.getStatusCode()) {
            return;
        }
        boolean existUploadCrawlerVideo = crawlerVideoService.existUploadCrawlerVideo(matchVideo.getContentId());
        log.info("processUploadCrawlerVideo contentId={} existCrawlerVideo={}", matchVideo.getContentId(), existUploadCrawlerVideo);
        if (!existUploadCrawlerVideo) {
            //已经存在结果  直接设置
            if (existMatchVideoResult(matchVideo)) {
                return;
            }
            String lockKey = String.format(UPLOAD_CRAWLER_LOCK_KEY, matchVideo.getContentId());
            String lockValue = UUID.randomUUID().toString();
            boolean lock = redisLock.tryLock(lockKey, lockValue, 20, TimeUnit.MINUTES);
            if (lock) {
                boolean res = crawlerVideoService.uploadCrawlerVideo(matchVideo.getContentId(), matchVideo.getFlowPoolLevel());
                if (res) {
                    successMatchVideo(matchVideo);
                    log.info("uploadCrawlerVideo success contentId={}", matchVideo.getContentId());
                } else {
                    log.info("uploadCrawlerVideo fail contentId={}", matchVideo.getContentId());
                    //上传失败记录
                    String countKey = String.format(UPLOAD_CRAWLER_FAIL_COUNT_KEY, matchVideo.getContentId());
                    Integer count = (Integer) redisTemplate.opsForValue().get(countKey);
                    if (count != null && count >= 3) {
                        LarkRobotUtil.sendMessage("crawlerVideoService uploadCrawlerVideo fail contentId=" + matchVideo.getContentId());
                        log.error("crawlerVideoService uploadCrawlerVideo fail contentId={}", matchVideo.getContentId());
                        //更新状态为失败
                        failMatchVideo(matchVideo);
                    } else {
                        if (count == null) {
                            redisTemplate.opsForValue().set(countKey, 1, 3, TimeUnit.DAYS);
                        } else {
                            redisTemplate.opsForValue().set(countKey, count + 1, 3, TimeUnit.DAYS);
                        }
                    }
                }
                redisLock.unlock(lockKey, lockValue);
            }
        } else {
            //更新状态为etl执行完成 并更新匹配成功状态
            successMatchVideo(matchVideo);
        }
    }

    public void publishCrawlerVideo() {
        List<MatchVideo> matchVideos;
        Integer id = (Integer) redisTemplate.opsForValue().get("last_match_video_id");
        if (id != null) {
            do {
                MatchVideoExample example = new MatchVideoExample();
                example.createCriteria().andIdGreaterThan(id)
                        .andContentStatusEqualTo(ContentStatusEnum.SUCCESS_3.getStatusCode())
                        .andPublishFlagEqualTo(PublicFlagEnum.PUBLIC.getStatusCode());
                example.setOrderByClause("id asc");
                Page<Object> page = new Page<>();
                page.setCurrentPage(1);
                page.setPageSize(1000);
                example.setPage(page);
                matchVideos = matchVideoMapper.selectByExample(example);
                if (CollectionUtils.isEmpty(matchVideos)) {
                    break;
                }
                id = matchVideos.get(matchVideos.size() - 1).getId();
                for (MatchVideo matchVideo : matchVideos) {
                    processPublishCrawlerVideo(matchVideo);
                }
            } while (!CollectionUtils.isEmpty(matchVideos));
        }
    }

    public void processPublishCrawlerVideo(MatchVideo matchVideo) {
        List<VideoDetail> videoDetails = contentService.getPublishVideoDetail(matchVideo, PublicFlagEnum.PUBLIC.getStatusCode(), null, null);
        if (CollectionUtils.isEmpty(videoDetails)) {
            return;
        }
        JSONArray response = getResponse(videoDetails);
        matchVideo.setContentStatus(ContentStatusEnum.SUCCESS_4.getStatusCode());
        matchVideo.setResponse(response.toJSONString());
        matchVideoMapper.updateByPrimaryKeySelective(matchVideo);
    }

    private JSONArray getResponse(List<VideoDetail> videoDetails) {
        JSONArray jsonArray = new JSONArray();
        for (VideoDetail videoDetail : videoDetails) {
            //生成match表json数据
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("uid", videoDetail.getUid());
            jsonObject.put("source", videoDetail.getSource());
            jsonObject.put("kimiTitle", videoDetail.getKimiTitle());
            jsonObject.put("videoId", videoDetail.getVideoId());
            jsonObject.put("videoCover", videoDetail.getShareImgPath());
            jsonObject.put("videoPath", videoDetail.getVideoPath());
            jsonObject.put("videoOss", videoDetail.getVideoOss());
            jsonArray.add(jsonObject);
        }
        return jsonArray;
    }

    public void updateStatus(Integer id, Integer status) {
        MatchVideo matchVideo = new MatchVideo();
        matchVideo.setId(id);
        matchVideo.setContentStatus(status);
        matchVideoMapper.updateByPrimaryKeySelective(matchVideo);
    }

    public boolean existMatchVideoResult(MatchVideo matchVideo) {

        //先查询缓存  缓存中存在直接返回结果
        String key = String.format(EXIST_RESULT_KEY, matchVideo.getContentId());
        Integer status = (Integer) redisTemplate.opsForValue().get(key);
        if (status != null) {
            updateStatus(matchVideo.getId(), status);
            return true;
        }

        //查询文章状态 查询到更新缓存
        LongArticlesTextExample example = new LongArticlesTextExample();
        example.createCriteria().andContentIdEqualTo(matchVideo.getContentId());
        List<LongArticlesText> longArticlesTexts = longArticlesTextMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(longArticlesTexts)) {
            return false;
        }
        LongArticlesText longArticlesText = longArticlesTexts.get(0);
        if (ContentResultStatusEnum.SUCCESS.getStatus() == longArticlesText.getResultStatus()) {
            updateStatus(matchVideo.getId(), ContentStatusEnum.SUCCESS_3.getStatusCode());
            redisTemplate.opsForValue().set(key, ContentStatusEnum.SUCCESS_3.getStatusCode(), 1, TimeUnit.DAYS);
            return true;
        }
        if (ContentResultStatusEnum.FAIL.getStatus() == longArticlesText.getResultStatus()) {
            updateStatus(matchVideo.getId(), ContentStatusEnum.ERROR_99.getStatusCode());
            redisTemplate.opsForValue().set(key, ContentStatusEnum.ERROR_99.getStatusCode(), 1, TimeUnit.DAYS);
            return true;
        }
        return false;
    }

    private void addStartProcessingTime(String contentId) {
        LongArticlesTextExample example = new LongArticlesTextExample();
        example.createCriteria().andContentIdEqualTo(contentId);
        List<LongArticlesText> longArticlesTexts = longArticlesTextMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(longArticlesTexts)) {
            return;
        }
        LongArticlesText longArticlesText = longArticlesTexts.get(0);
        if (longArticlesText.getStartProcessingTime() == null) {
            LongArticlesText updateLongArticlesText = new LongArticlesText();
            updateLongArticlesText.setContentId(contentId);
            updateLongArticlesText.setStartProcessingTime(new Date());
            longArticlesTextMapper.updateByPrimaryKeySelective(updateLongArticlesText);
        }
    }

    private void updateResultStatus(String contentId, Integer resultStatus) {
        LongArticlesText updateLongArticlesText = new LongArticlesText();
        updateLongArticlesText.setContentId(contentId);
        updateLongArticlesText.setResultStatus(resultStatus);
        longArticlesTextMapper.updateByPrimaryKeySelective(updateLongArticlesText);

    }

    private void failMatchVideo(MatchVideo matchVideo) {
        updateStatus(matchVideo.getId(), ContentStatusEnum.ERROR_99.getStatusCode());
        updateResultStatus(matchVideo.getContentId(), ContentResultStatusEnum.FAIL.getStatus());
    }

    private void successMatchVideo(MatchVideo matchVideo) {
        updateStatus(matchVideo.getId(), ContentStatusEnum.SUCCESS_3.getStatusCode());
        updateResultStatus(matchVideo.getContentId(), ContentResultStatusEnum.SUCCESS.getStatus());
    }

    public void updateRovn() {
        PiaoquanSourceVideoPoolExample example = new PiaoquanSourceVideoPoolExample();
        example.createCriteria().andStatusEqualTo(2);
        List<PiaoquanSourceVideoPool> piaoquanSourceVideoPools = piaoquanSourceVideoPoolMapper.selectByExample(example);
        final int BATCH_SIZE = 100;
        for (int i = 0; i < piaoquanSourceVideoPools.size(); i += BATCH_SIZE) {
            List<PiaoquanSourceVideoPool> batch = piaoquanSourceVideoPools.subList(
                    i,
                    Math.min(i + BATCH_SIZE, piaoquanSourceVideoPools.size())
            );
            if (!CollectionUtils.isEmpty(batch)) {
                StringBuilder inClause = new StringBuilder();
                inClause.append("(");
                for (int j = 0; j < batch.size(); j++) {
                    inClause.append("'").append(batch.get(j).getVideoId()).append("'");
                    if (j < batch.size() - 1) {
                        inClause.append(",");
                    }
                }
                inClause.append(")");
                String sqlStr = "SELECT * " +
                        "FROM ( " +
                        "    SELECT *, " +
                        "           ROW_NUMBER() OVER (PARTITION BY 视频id ORDER BY dt DESC) AS rn " +
                        "    FROM loghubods.wecom_cooperation_video_candidate_pool " +
                        "    WHERE dt <= %s " +
                        "      AND 视频id IN %s " +
                        ") t " +
                        "WHERE rn = 1;";
                String sql = String.format(sqlStr, DateUtil.getBeforeDayDateString("yyyyMMdd"), inClause);
                List<Record> recordList = OdpsUtil.getOdpsData(sql);
                if (CollectionUtils.isEmpty(recordList)) {
                    continue;
                }
                Map<Integer, Double> scoreMap = new HashMap<>();
                for (Record record : recordList) {
                    Integer videoId = Integer.parseInt((String) record.get(0));
                    Double score = Double.parseDouble((String) record.get(4));
                    scoreMap.put(videoId, score);
                }
                for (PiaoquanSourceVideoPool pool : batch) {
                    Double score = scoreMap.get(pool.getVideoId());
                    if (score != null) {
                        Integer id = pool.getId();
                        PiaoquanSourceVideoPool updatePiaoquanSourceVideoPool = new PiaoquanSourceVideoPool();
                        updatePiaoquanSourceVideoPool.setId(id);
                        updatePiaoquanSourceVideoPool.setRovn(score);
                        piaoquanSourceVideoPoolMapper.updateByPrimaryKeySelective(updatePiaoquanSourceVideoPool);
                    }
                }
            }
        }
    }
}



