package com.tzld.piaoquan.longarticle.service.local.impl;

import com.tzld.piaoquan.longarticle.common.enums.NewContentStatusEnum;
import com.tzld.piaoquan.longarticle.dao.mapper.aigc.ProducePlanExeRecordMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.LongArticlesTextMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.NewMatchVideoMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.PromotionSourceMapper;
import com.tzld.piaoquan.longarticle.model.bo.MatchContent;
import com.tzld.piaoquan.longarticle.model.po.aigc.ProducePlanExeRecord;
import com.tzld.piaoquan.longarticle.model.po.aigc.ProducePlanExeRecordExample;
import com.tzld.piaoquan.longarticle.model.po.longarticle.*;
import com.tzld.piaoquan.longarticle.model.vo.MatchContentItem;
import com.tzld.piaoquan.longarticle.service.local.KimiService;
import com.tzld.piaoquan.longarticle.service.local.NewMatchVideoService;
import com.tzld.piaoquan.longarticle.service.remote.AigcService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.function.Consumer;

import static com.tzld.piaoquan.longarticle.component.ThreadPools.*;

@Slf4j
@Service
public class NewMatchVideoServiceImpl implements NewMatchVideoService {

    @Autowired
    KimiService kimiService;

    @Autowired
    private CrawlerVideoServiceImpl crawlerVideoService;

    @Autowired
    private ProducePlanExeRecordMapper producePlanExeRecordMapper;

    @Autowired
    private PromotionSourceMapper promotionSourceMapper;

    @Autowired
    private NewMatchVideoMapper newMatchVideoMapper;

    @Autowired
    private LongArticlesTextMapper longArticlesTextMapper;

    @Autowired
    private AigcService aigcService;


    @Transactional
    @Override
    public void addMatchVideo(MatchContentItem matchContentItem) {
        NewMatchVideoExample example = new NewMatchVideoExample();
        example.createCriteria().andContentIdEqualTo(matchContentItem.getProduceContentId());
        long l = newMatchVideoMapper.countByExample(example);
        if (l > 0) {
            return;
        }
        NewMatchVideo newMatchVideo = new NewMatchVideo();
        newMatchVideo.setContentId(matchContentItem.getProduceContentId());
        newMatchVideo.setFlowPoolLevel(matchContentItem.getFlowPoolLevelTag());
        newMatchVideo.setContentStatus(NewContentStatusEnum.DEFAULT.getStatusCode());
        newMatchVideoMapper.insertSelective(newMatchVideo);
        LongArticlesTextExample longArticlesTextExample = new LongArticlesTextExample();
        longArticlesTextExample.createCriteria().andContentIdEqualTo(matchContentItem.getProduceContentId());
        long l1 = longArticlesTextMapper.countByExample(longArticlesTextExample);
        if (l1 == 0) {
            LongArticlesText longArticlesText = new LongArticlesText();
            longArticlesText.setArticleTitle(matchContentItem.getTitle());
            longArticlesText.setArticleText(matchContentItem.getContent());
            longArticlesText.setContentId(matchContentItem.getProduceContentId());
            longArticlesText.setKimiStatus(0);
            longArticlesTextMapper.insertSelective(longArticlesText);
        }
    }


    // 抽取公共的查询逻辑
    private List<NewMatchVideo> queryMatchVideosByStatus(NewContentStatusEnum status) {
        NewMatchVideoExample example = new NewMatchVideoExample();
        example.createCriteria().andContentStatusEqualTo(status.getStatusCode());
        return newMatchVideoMapper.selectByExample(example);
    }

    // 抽取公共的并行处理逻辑
    private void processVideosInParallel(List<NewMatchVideo> videos,
                                         Consumer<NewMatchVideo> processor,
                                         ExecutorService executor) throws InterruptedException {
        if (CollectionUtils.isEmpty(videos)) {
            return;
        }

        CountDownLatch countDownLatch = new CountDownLatch(videos.size());
        for (NewMatchVideo video : videos) {
            executor.execute(() -> {
                try {
                    processor.accept(video);
                } finally {
                    countDownLatch.countDown();
                }
            });
        }

        countDownLatch.await();
    }

    // 抽取公共的状态更新逻辑
    private void updateVideoStatus(NewMatchVideo video, boolean success,
                                   NewContentStatusEnum successStatus,
                                   NewContentStatusEnum failStatus) {
        int retryCount = video.getRetryCount();
        int newStatus;

        if (success) {
            newStatus = successStatus.getStatusCode();
            retryCount = 0;
        } else {
            retryCount++;
            newStatus = (retryCount > 3)
                    ? failStatus.getStatusCode()
                    : video.getContentStatus();
        }

        video.setContentStatus(newStatus);
        video.setRetryCount(retryCount);

        try {
            newMatchVideoMapper.updateByPrimaryKeySelective(video);
        } catch (Exception e) {
            log.error("Failed to update video status for ID: {}", video.getContentId(), e);
        }
    }

    @Override
    public void kimiContent() throws InterruptedException {
        List<NewMatchVideo> videos = queryMatchVideosByStatus(NewContentStatusEnum.DEFAULT);
        processVideosInParallel(videos,
                this::processKimiMatchContent,
                newMatchKimiVideoPoolExecutor);
    }

    @Override
    public void matchCrawlerVideo() throws InterruptedException {
        List<NewMatchVideo> videos = queryMatchVideosByStatus(NewContentStatusEnum.KIMI_SUCCESS);
        processVideosInParallel(videos,
                this::processCrawlerMatchContent,
                newMatchCrawlerVideoPoolExecutor);
    }

    @Override
    public void uploadCrawlerVideo() throws InterruptedException {
        List<NewMatchVideo> videos = queryMatchVideosByStatus(NewContentStatusEnum.CRAWLER_SUCCESS);
        processVideosInParallel(videos,
                this::processUploadCrawlerVideo,
                newUploadCrawlerVideoPoolExecutor);
    }

    private void processKimiMatchContent(NewMatchVideo newMatchVideo) {
        boolean updateSuccess = false;
        try {
            updateSuccess = kimiService.updateKimiContent(newMatchVideo.getContentId());
        } catch (Exception e) {
            log.error("Failed to update Kimi content for ID: {}", newMatchVideo.getContentId(), e);
        }

        updateVideoStatus(newMatchVideo, updateSuccess,
                NewContentStatusEnum.KIMI_SUCCESS,
                NewContentStatusEnum.KIMI_FAIL);
    }

    public void processCrawlerMatchContent(NewMatchVideo newMatchVideo) {
        boolean updateSuccess = false;
        try {
            LongArticlesText kimiText = kimiService.getKimiText(newMatchVideo.getContentId());
            String rootContentId = getRootContentId(newMatchVideo.getContentId());
            updateSuccess = crawlerVideoService.addCrawlerVideo(newMatchVideo.getContentId(), rootContentId, kimiText);
        } catch (Exception e) {
            log.error("Failed to crawler match video content for ID: {}", newMatchVideo.getContentId(), e);
        }

        updateVideoStatus(newMatchVideo, updateSuccess,
                NewContentStatusEnum.CRAWLER_SUCCESS,
                NewContentStatusEnum.CRAWLER_FAIL);
    }

    private void processUploadCrawlerVideo(NewMatchVideo newMatchVideo) {
        boolean updateSuccess = false;
        try {
            updateSuccess = crawlerVideoService.uploadCrawlerVideo(newMatchVideo.getContentId(), newMatchVideo.getFlowPoolLevel());
        } catch (Exception e) {
            log.error("Failed to upload crawler video content for ID: {}", newMatchVideo.getContentId(), e);
        }

        updateVideoStatus(newMatchVideo, updateSuccess,
                NewContentStatusEnum.ELT_SUCCESS,
                NewContentStatusEnum.ELT_FAIL);

    }

    private String getRootContentId(String contentId) {
        ProducePlanExeRecordExample example = new ProducePlanExeRecordExample();
        example.createCriteria().andPlanExeIdEqualTo(contentId);
        List<ProducePlanExeRecord> producePlanExeRecords = producePlanExeRecordMapper.selectByExample(example);
        if (!CollectionUtils.isEmpty(producePlanExeRecords)) {
            String channelContentId = producePlanExeRecords.get(0).getChannelContentId();
            if (channelContentId == null) {
                return null;
            }
            PromotionSourceExample promotionSourceExample = new PromotionSourceExample();
            promotionSourceExample.createCriteria().andChannelContentIdEqualTo(channelContentId);
            List<PromotionSource> promotionSources = promotionSourceMapper.selectByExample(promotionSourceExample);
            if (CollectionUtils.isEmpty(promotionSources)) {
                return null;
            }
            return promotionSources.get(0).getRootProduceContentId();
        }
        return null;
    }


}