package com.tzld.piaoquan.longarticle.utils.other;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.longarticle.utils.ConfigUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.*;
import java.nio.file.Files;
import java.util.Objects;
import java.util.UUID;

import static com.tzld.piaoquan.longarticle.common.constants.ProxyConstant.*;

@Slf4j
public class VideoDownloader {

    private static final long READ_TIMEOUT_SECONDS = 60; // 读取超时时间（秒）

    private static final int MAX_RETRIES = 3;

    public static String downloadCover(String outVideoId, String platform, String coverUrl) {
        String path = generateCoverPath(platform, outVideoId);
        if (download(path, coverUrl, platform) == 0) {
            return path;
        }
        return "";
    }

    public static String downloadVideo(String outVideoId, String platform, String videoUrl) {
        int retries = 0;
        while (retries < MAX_RETRIES) {
            log.info("downloadVideo outVideoId={} retries={}", outVideoId, retries);
            int download = -1;
            if (StringUtils.isNotEmpty(videoUrl)) {
                String path = generateVideoPath(platform, outVideoId);
                download = download(path, videoUrl, platform);
                if (download == 0) {
                    return path;
                }
            }
            //百度重新获取链接
            if ((download == 403 || StringUtils.isEmpty(videoUrl)) && Objects.equals(platform, "baidu_search")) {
                JSONObject videoDetail = HkspSearch.getVideoDetail(outVideoId);
                if (videoDetail != null) {
                    String newVideoUrl = videoDetail.getString("playurl");
                    if (newVideoUrl != null) {
                        videoUrl = newVideoUrl;
                    }
                }
            }

            //抖音重新获取链接
            if ((download == 403 || StringUtils.isEmpty(videoUrl)) && Objects.equals(platform, "dy_search")) {
                JSONObject videoDetail = DouyinSearch.douyinDetail(outVideoId);
                if (videoDetail != null) {
                    JSONArray videoUrlList = videoDetail.getJSONArray("video_url_list");
                    if (!CollectionUtils.isEmpty(videoUrlList)) {
                        String newVideoUrl = videoUrlList.getJSONObject(0).getString("video_url");
                        if (newVideoUrl != null) {
                            videoUrl = newVideoUrl;
                        }
                    }
                }
            }
            retries++;
        }
        return "";
    }

    public static int download(String path, String videoUrl, String platform) {
        try {
            log.info("download video path={}", path);
            if (StringUtils.isEmpty(videoUrl)) {
                return -1;
            }
            File file = new File(path);
            if (file.exists()) {
                file.delete();
            }
            Proxy proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(PROXY_HOST, PROXY_PORT));
            HttpURLConnection connection = (HttpURLConnection) new URL(videoUrl).openConnection(proxy);
            connection.setConnectTimeout(10000);
            // 设置读取超时时间（单位：毫秒）
            connection.setReadTimeout(60000);
            connection.setRequestMethod("GET");
            connection.setRequestProperty("User-Agent", "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36 Edg/131.0.0.0");
            connection.setRequestProperty("Accept", "*/*");
            connection.setRequestProperty("accept-language", "en,zh;q=0.9,zh-CN;q=0.8");
            if (Objects.equals(platform, "dy_search")) {
                connection.setRequestProperty("referer", "https://v11-coldf.douyinvod.com/");
            }
            // 连接并获取响应
            connection.connect();

            int responseCode = connection.getResponseCode();
            if (responseCode == HttpURLConnection.HTTP_OK || responseCode == HttpURLConnection.HTTP_PARTIAL) {
                long expectedSize = connection.getContentLengthLong();
                InputStream inputStream = connection.getInputStream();
                OutputStream outputStream = Files.newOutputStream(file.toPath());
                byte[] buffer = new byte[1024 * 1024];
                int bytesRead;
                log.info("download start path={} expectedSize={}", path, expectedSize);
                long size = 0;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    size += bytesRead;
                    outputStream.write(buffer, 0, bytesRead);
                }
                log.info("download end path={}", path);
                if (file.length() != 0) {
                    return 0;
                }
            }
            return responseCode;
        } catch (Exception e) {
            log.error("download error", e);
        }
        return -1;
    }

    private static String generateVideoPath(String platform, String videoId) {
        String index = String.format("%s-%s-%s", platform, videoId, UUID.randomUUID());
        String md5Hash = DigestUtils.md5Hex(index);
        String fileName = String.format("%s.mp4", md5Hash);
        String path = ConfigUtil.getConfig("download.path");
        File uploadDir = new File(path);
        if (!uploadDir.exists()) {
            uploadDir.mkdirs(); // 创建目录
        }
        return String.join(File.separator, path, fileName); // 返回文件路径和封面路径
    }

    private static String generateCoverPath(String platform, String videoId) {
        String index = String.format("%s-%s-%s", platform, videoId, UUID.randomUUID());
        String md5Hash = DigestUtils.md5Hex(index);
        String coverName = String.format("%s.png", md5Hash);
        String path = ConfigUtil.getConfig("download.path");
        File uploadDir = new File(path);
        if (!uploadDir.exists()) {
            uploadDir.mkdirs(); // 创建目录
        }
        return String.join(File.separator, path, coverName); // 返回文件路径和封面路径
    }

    public static void main(String[] args) {
        System.setProperty("jdk.http.auth.tunneling.disabledSchemes", "");
        Authenticator.setDefault(
                new Authenticator() {
                    public PasswordAuthentication getPasswordAuthentication() {
                        return new PasswordAuthentication(
                                USERNAME, PASSWORD.toCharArray());
                    }
                }
        );
        downloadVideo("2562802497357206175", "baidu_search", "https://vdept3.bdstatic.com/mda-qkt120240g2y8uka/cae_h264/1732792181901415854/mda-qkt120240g2y8uka.mp4?v_from_s=hkapp-haokan-suzhou&auth_key=1738759058-0-0-115429d241bdab4208d998b5f3f95507&bcevod_channel=searchbox_feed&cr=0&cd=0&pd=1&pt=3&logid=2258462520&vid=2562802497357206175&abtest=132219_1");
    }

}